/** Presenter class
 *
 * @class Presenter
 * 
 *
 * @author <a href="mailto:info@ilevia.com">info@ilevia.com</a>
 *
 *
 * @required <a href="http://jquery.com/">jQuery</a>
 * @required <a href="http://jqueryui.com/">jQuery UI</a>
 *
 */
//"use strict";

function Presenter(){
    //Build singleton instance
    if ( arguments.callee._singletonInstance ){
        console.warn("Singleton instance used");
        return arguments.callee._singletonInstance;
    }
    arguments.callee._singletonInstance = this;

	var self = this;

    /** App JSON data informations (eg. version, application name)
     * @private
     * @type {undefined}
     */
	var informations = undefined;
	
	/** Application dictionary instance
	 * @type {Dictionary|*}
	 */
    this.Dictionary = new Dictionary();
    
    /** Application resources manager controller
     * @type ResoucesManagerController
     */
    this.ResoucesManager = new ResourcesController( model["instance"].ResourcesManager );
    
    /** Application data point types (DPT) for KNX protocol
     * @type Array
     */
    this.DPT = undefined;
    
    // Methods
    
    /** Presenter initialization
     * 
     */
    this.init = function(){






        console.log("%cPresenter class initialization","color:green");
        
        //Add view initialization event on document ready
        document.addEventListener("DOMContentLoaded",initView);

        //Load and fill protocols combo box from "/protocols/declare.json"
        $.ajax({ dataType: "json", url: "./info/info.json", async: false }).success(function(oData){
            informations = oData;

            presenter.preferences.setApplicationPreference("name",informations["name"]);
            presenter.preferences.setApplicationPreference("version",informations["version"]);
        }).error(function(jqXHR,textStatus,errorThrown){
            alert("Fatal error: Failed to load Application info");
            QT.exit();
        });
        
        //Load and fill data point type (DPT) "/declarations/DataPointTypes.json"
        $.ajax({ dataType: "json", url: "./declarations/DataPointTypes.json" }).success(function(oData){
            //console.log(oData);
            self.DPT = oData;
        });
        
        //Load and fill protocols combo box from "/protocols/declare.json"
        /*$.ajax({ dataType: "json", url: "../config.json", async: false, cache: false }).success(function(oData){
            //Exist
            $.extend(true,config,oData);
        }).error(function(jqXHR,textStatus,errorThrown){
            //Don't exist
            console.warn("Configuration file \"%c./config.json%c\" initialized","color:orange;","color:inherit");
            
            for(var k in config){
                setConfigParam(k,config[k]);
            }
        });*/

        //Overwrite onLoadDictionary event function
        self.Dictionary.onLoadDictionary = function(oJson){
            //setConfigParam("language",oJson["language-id"]);
            presenter.preferences.setParameter("language",oJson["language-id"]);

            // Translate UI
            var $dictionary = $("[dictionary]");
            var $titles = $("[dictionary-title]");
            
            for(var i=0; i<$dictionary.length; ++i){
                var $current = $($dictionary[i]);
                var $next = $current;
                var $target = $current;
                
                //Check inner children
                while($next.length){
                    $target = $next;
                    $next = $next.children();
                }
                
                $target.text(self.Dictionary.getName($current.attr("dictionary")));
            }
            
            for(var i=0; i<$titles.length; ++i){
                var $current = $($titles[i]);
                $current.attr("title",self.Dictionary.getName($current.attr("dictionary-title")));
            }
        };

        self.Dictionary.setLanguage( presenter.preferences.getParameter("language") );

        //Resize subscription
        $.subscribe("/view/window/resize",PubSub.View.Resize.Window);
        $.subscribe("/view/interface/resize",PubSub.View.Resize.Interface);
        $.subscribe("/view/lists/resize",PubSub.View.Resize.Lists);
        $.subscribe("/view/relations/resize",PubSub.View.Resize.Relations);
        $.subscribe("/view/interface/maps/resize",PubSub.View.Resize.Maps);
        $.subscribe("/view/components/resize",PubSub.View.Resize.Components);
        $.subscribe("/view/gateways/resize",PubSub.View.Resize.Gateways);
        
        //Configuration subscription
        $.subscribe("/view/application/refresh/parameter",PubSub.View.Refresh.Application.Parameter);
        
        //Resources subscription
        $.subscribe("/view/application/refresh/icons",PubSub.View.Refresh.Application.Icons);

        //Resources subscription
        $.subscribe("/view/application/refresh/connection",PubSub.View.Refresh.Application.Connection);
        
        //Set current project to undefined
        presenter.preferences.setParameter("currentProject",undefined);
        presenter.preferences.setParameter("connectedUser",undefined);
        presenter.preferences.setParameter("currentUser",undefined);

        // Init model
        initModel();
    };
    
	/** Presenter initialize Model
	 * 
	 */
	var initModel = function(){
        console.log("Init Model");
        $.ajax({

            dataType: "json",
            url: "./components/Skin.json",
            async: false ,
            success:function(aData)
            {
                console.log(aData);
                model["instance"].skin = aData;

            },
            error:function(jqXHR,textStatus,errorThrown) {
                console.error(textStatus);
            }
        });

        $.ajax({

            dataType: "json",
            url: "./info/datatype.json",
            async: false ,
            success:function(aData)
            {
                console.log(aData);
                model["instance"].datatype = aData;

            },
            error:function(jqXHR,textStatus,errorThrown) {
                console.error(textStatus);
            }
        });

        //Load and fill protocols combo box from "/protocols/declare.json"
        $.ajax({ dataType: "json", url: "./declarations/protocols.json", async: false }).success(function(aData){
            model["instance"].protocols = aData;
            for(var i=0; i<aData.length; ++i){
                model["instance"].protocolsLabel[aData[i].id] = aData[i].label;
            }
        });

	    // Set application info to project (eg. version, application name)
	    eveManager.version = informations.version;
	    eveManager.name = informations.name;

		console.log("Get icons pan");
		var pans=model["instance"].skin.IconsPan;
		for(var i=0; i<pans.length; ++i){
			console.log(pans[i]);
			self.ResoucesManager.addIconsPan(pans[i]);
        }

        // carica icons pan da storage
        var customIcons=new Array();
        try{
            customIcons=JSON.parse( QT.readAbsoluteFile(QT.getStoragePath()+"/assets/icons/custom/iconsPan.json"));

        }
        catch (e){customIcons=new Array();}


        for (var i=0;i<customIcons.length;i++)
        {
            application.ResoucesManager.addIconsPan(customIcons[i]);
        }


/*
 application.ResourcesManager.refreshIconsFromStorage();
 self.ResourcesManager().addIconsFromModel();

 */


/*
        self.ResoucesManager.addIconsPan({
            "id" : "EVEIcons",

            "label": "EVE Icons",
            "name": "eve-icons.png",
            "visible": true,
            "description": "Eve Remote icons",
            "src": "./assets/icons/ilevia/eve-icons.png",
            "srcSelected": "./assets/icons/ilevia/selected_eve-icons.png",
            "imgHeight": 2000,
            "imgWidth": 1000,
            "iconsHeight": 102,
            "iconsWidth": 100
        });
  */         


/*



        self.ResoucesManager.addIconsPan({
            "id" : "2",
            "label": "GEWISS Icons",
            "name": "gewiss.png",
            "visible": true,
            "description": "GEWISS icons",
            "src": "./assets/icons/gewiss/gewiss.png",
            "srcSelected": "",
            "imgHeight": 2000,
            "imgWidth": 1000,
            "iconsHeight": 102,
            "iconsWidth": 100
        });
*/

        /*self.ResoucesManager.addIconsPan({
            "name": "Ilevia - new icons",
            "visible": true,
            "description": "Eve Remote new icons",
            "src": "./assets/icons/ilevia/icons_menu_up2.png",
            "imgHeight": 1632,
            "imgWidth": 1000,
            "iconsHeight": 102,
            "iconsWidth": 100
        });*/
        /*
        self.ResoucesManager.addIconsPan({
            "name": "Ilevia iPad",
            "visible": false,
            "description": "Eve Remote icons",
            "src": "./assets/icons/ilevia/eve-icons-big.png",
            "imgHeight": 1224,
            "imgWidth": 1500,
            "iconsHeight": 153,
            "iconsWidth": 150
        });

        self.ResoucesManager.addIconsPan({
            "name": "Ilevia - components",
            "visible": true,
            "description": "Eve Remote icons for mobile tablet (status 0)",
            "src": "./assets/icons/ilevia/eve-components-48x48.png",
            "imgHeight": 192,
            "imgWidth": 384,
            "iconsHeight": 48,
            "iconsWidth": 48
        });

        self.ResoucesManager.addIconsPan({
            "name": "Ilevia - components",
            "visible": false,
            "description": "Eve Remote icons for mobile",
            "src": "./assets/icons/ilevia/eve-components-32x32.png",
            "imgHeight": 128,
            "imgWidth": 256,
            "iconsHeight": 32,
            "iconsWidth": 32
        });
        
        self.ResoucesManager.addIconsPan({
            "name": "Test",
            "visible": true,
            "description": "Eve Remote icons for mobile",
            "src": "./assets/icons/custom/icons-preview.png",
            "imgHeight": 900,
            "imgWidth": 1033,
            "iconsHeight": 60,
            "iconsWidth": 80
        });*/

        //Add admin user
        var oAdminNode = self.addUser({
            name: "admin",
            password: "5f4dcc3b5aa765d61d8327deb882cf99",
            description: self.Dictionary.getName("System administrator")//,
            //createdBy: "ilevia"
        });

        var oAdmin = oAdminNode.getData();

        oAdmin.logoutIcon.srcId = self.ResoucesManager.getDefaultIconsPan()["id"];

        //Load and fill protocols combo box from "/protocols/declare.json"
        /*$.ajax({ dataType: "json", url: "./declarations/userPermissions.json", async: false }).success(function(aData){
            for(var i=0; i<aData.length; ++i){
                $.extend(true, aData[i], { enabled: true });

                oAdmin.addPermission(aData[i]["id"], aData[i]);
            }
        });*/

        //TODO IMPORTANT: before loading project ask user password and after set model["user"] with logged user
        application.connectUser( oAdminNode );

        //self.selectUser(oAdminNode);
        
		// Ghost gateway creation
		var ghostGatewayParam = {
			name: "Eve",
			protocol: "ghost",
			UId: "eve-ghost",
			special: "ghost"
		};
		var oGhostNode = self.addGateway(ghostGatewayParam);
		var oGhost = oGhostNode.getData();
		
		// Multimedia gateway creation (for video Cameras)
        var videoGatewayParam = {
            name: "Multimedia",
            protocol: "video",
            UId: "eve-multimedia",
            special: "video"
        };
        var oVideoNode = self.addGateway(videoGatewayParam);
        
        // Web gateway creation (for web pages)
        var webGatewayParam = {
            name: "Web",
            protocol: "url",
            UId: "eve-web",
            special: "web"
        };
        var oVideoNode = self.addGateway(webGatewayParam);
        
        // Gui gateway creation (for widget)
        var guiGatewayParam = {
            name: "Graphic UI",
            protocol: "gui",
            UId: "eve-gui",
            special: "gui"
        };
        var oGuiNode = self.addGateway(guiGatewayParam);
		
		// Select gateway
		self.selectGateway(oGhostNode);
		
		// Extra / Timing / Wait component creation (AGGIORNARE)
		var componentExtraParam = {
			name: self.Dictionary.getName("Extra"),
			type: "group",
			UId: "eve-extra",
			special: "extra"
		};
		var oComponentExtra = self.addComponent(componentExtraParam);
		var componentWaitParam = {
			name: self.Dictionary.getName("Wait"),
			type: "wait",
			UId: "eve-wait",
			special: "wait"
		};
		var oComponentWait = self.addComponent(componentWaitParam,oComponentExtra);
        var componentLabelParam = {
			name: self.Dictionary.getName("Label"),
			type: "label",
			UId: "eve-label",
			special: "label"
		};

        var oComponentLabel = self.addComponent(componentLabelParam);

        var componentShellParam = {
            name: self.Dictionary.getName("Shell"),
            type: "shell",
            UId: "999999",
            special: "extra",
            cache: true
        };

        var oComponentLabel = self.addComponent(componentShellParam);
        // restart component id number
        model.instance.ComponentUniqueIdAssign.setId("10");
        //self.addComponent("switch");
		// Select component
		
		// Main subscription
		/*$.subscribe("/view/gateways/refresh/properties",PubSub.View.Refresh.Gateway.Properties);
		$.subscribe("/view/components/refresh/properties",PubSub.View.Refresh.Component.Properties);
		$.subscribe("/view/components/refresh/dependencies",PubSub.View.Refresh.Component.Dependencies);
		$.subscribe("/view/uicomponents/refresh/dependencies",PubSub.View.Refresh.UIComponent.Dependencies);*/

		//Clear project directory
        //QT.clearProject();

	};
	
	/** Presenter initialize View
     * 
     */
	var initView = function(){
	    console.log("%cBuild view operations","color:brown;");
	    
	    //$document.trigger("app/load/begin",["Initialization"]);
	    
	    //Switch view to ready
	    View.ready = true;
	    
	    //Set language
	    self.Dictionary.setLanguage(presenter.preferences.getParameter("language"));

	    // Root and default node initialization
        $("#myinterface #level-1").data("eveManagerLinkedNode",eveManager.getUIComponentsRoot());
        $("#myinterface #level-1").data("eveManagerNode",eveManager.getUIComponentsRoot());
        $("#myinterface").data("eveManagerNode",eveManager.getUIComponentsRoot());
        $("#myinterface").data("eveManagerNode",eveManager.getUIComponentsRoot());

        $("#mylists #level-11").data("eveManagerLinkedNode",eveManager.getUIComponentsRoot());
        $("#mylists #level-11").data("eveManagerNode",eveManager.getUIComponentsRoot());
        $("#mylists").data("eveManagerNode",eveManager.getUIComponentsRoot());

        $("#componentscontainer").data("eveManagerNode",eveManager.getComponentsRoot());
        $("#mygateways").data("eveManagerNode",eveManager.getGatewaysRoot());

        //Set colpick presets
        var _colPresets = presenter.preferences.getParameter("colorPresets");
        console.log("presente precolor");
        var $colPresets = $(".colpick_hex .colpick_preset");
        console.log($colPresets);
        for(var i=0; i<_colPresets.length; ++i){
            var $li = $("<li></li>");
            var $color = $("<div></div>");
            var $saveBtn = $("<span class='ui-icon ui-icon-disk' title='save'></span>");

            $color.css("background-color",_colPresets[i]);
            $li.append($color);
            $li.append($saveBtn);
            $colPresets.append($li);
        }
	    
	    //View subscriptions
	    $.subscribe("/view/gateways/refresh/properties",PubSub.View.Refresh.Gateway.Properties);
        $.subscribe("/view/components/refresh/properties",PubSub.View.Refresh.Component.Properties);
        $.subscribe("/view/gateways/refresh/dependencies",PubSub.View.Refresh.Gateway.Dependencies);
        $.subscribe("/view/components/refresh/dependencies",PubSub.View.Refresh.Component.Dependencies);
        $.subscribe("/view/uicomponents/refresh/dependencies",PubSub.View.Refresh.UIComponent.Dependencies);
        $.subscribe("/view/imported/projects/knx/refresh/address",PubSub.View.Refresh.Imported.Projects.Knx.Address);

	    //Add created icons pan
        var aPans = model["instance"].ResourcesManager.getSpritesByType("pan/icons");
        //--console.log(aPans);
        for(var i=0; i<aPans.length; ++i){
            View.addIconsPan(aPans[i]);
        }





        //Add created users
        var aUsers = model["instance"].getUsersRoot();

        for(var i=0,l=aUsers.children.length; i<l; ++i){
            View.addUser(aUsers.children[i]);
        }

        //Select User
        View.selectUser( model["instance"].getSelectUser() );
	    
	    //Add created gateway
	    var oGatewaysRoot = eveManager.getGatewaysRoot();
        
        for(var i=0; i<oGatewaysRoot.children.length; ++i){
            var oGateway = oGatewaysRoot.children[i].getData();
            var strId = oGateway.id;
            var $element = View.addGateway(oGatewaysRoot.children[i]);
            
            $.publish("/view/gateways/"+ strId +"/refresh/name",[oGateway]);
            $.publish("/view/gateways/"+ strId +"/refresh/protocol",[oGateway]);
            $.publish("/view/gateways/"+ strId +"/refresh/parameter",[oGateway]);
        }
        
        //Add created components (only root level)
        var recursiveComponentBuilding = function(oNode,oParent){
            if(oNode.children.length < 1){
                var oComponent = oNode.getData();
                var strId = oComponent.id;
                var $element = View.addComponent(oNode,oParent);
                
                $.publish("/view/components/"+ strId +"/refresh/parameter",[oComponent,"name"]);
                $.publish("/view/components/"+ strId +"/refresh/parameter",[oComponent,"icons"]);
            }
            else{
                for(var i=0; i<oNode.children.length; ++i){
                    var oComponent = oNode.getData();
                    var strId = oComponent.id;
                    var $element = View.addComponent(oNode,oParent);
                    
                    $.publish("/view/components/"+ strId +"/refresh/parameter",[oComponent,"name"]);
                    $.publish("/view/components/"+ strId +"/refresh/parameter",[oComponent,"icons"]);
                    // ora i componenti vengono visualizzati quando si aprono i gruppi
                    // quindi non si generano i sottocomponenti della root
                    // recursiveComponentBuilding(oNode.children[i],oNode);
                }
            }
        };
        var oComponentsRoot = eveManager.getComponentsRoot();
        
        for(var i=0; i<oComponentsRoot.children.length; ++i){
            recursiveComponentBuilding(oComponentsRoot.children[i],oComponentsRoot);
        }


    /*    var strConnectWith = "#myinterface .level, #mylists .level,#interface-maps .maps-container .viewer";

        var strInterfaceElementSelector = "#interface-elements #interface-components li";
        $(strInterfaceElementSelector).draggable({
            appendTo: "body",
            containment: "body",
            revert: "invalid",
            revertDuration: false,
            connectToSortable: strConnectWith,
            //helper: "clone",
            helper: function(e){ return $(e.currentTarget).clone(true); },
            zIndex: 1000
        });
      */  //Select
        var oGatewaySelected = eveManager.getSelectGateway();
        View.selectGateway(oGatewaySelected);
        
        //$document.trigger("app/load/end");

        // da commetare in fase di debug

        $("#componentslibrary").find("[eve-manager-component='googlecalendar']").hide();
        $("#componentslibrary").find("[eve-manager-component='realtimechart']").hide();
        $("#componentslibrary").find("[eve-manager-component='listview']").hide();
        $("#componentslibrary").find("[eve-manager-component='AreaLightController']").hide();
        $("#componentslibrary").find("[eve-manager-component='dynamic']").hide();
        $("#componentslibrary").find("[eve-manager-component='seat']").hide();
	};

    /** Users */

    /** Add user
     * @returns {*} New selected node
     * @param oParam
     */
    this.addUser = function(oParam,oParentUser){

        //Check
        if(oParam == undefined){
            console.error("User name required.");
            return undefined;
        }

        var oDefaultParams = {
            /*permissions: [
                "describe params"
            ]*/
        };

        //Check input parameters
        if(typeof oParam == "string"){
            var name = oParam;
            oParam = { name: name };
        }

        //Extend default parameters
        $.extend(true,oParam,oDefaultParams);

        //Add gateway to Model
        var oNode ;
        try

        {
            oNode= model["instance"].addUser(oParam.name);

            //Initialize User


            var oUser = oNode.getData();
            oUser.logoutIcon.srcId = self.ResoucesManager.getDefaultIconsPan()["id"];


            var strId = oUser.id;

            //Add to parent User
            //if(oParentUser != undefined){
            if(oUser.name != "admin"){
                if(oParentUser instanceof Node){
                    oParentUser.getData().childsUsers.push(oNode);
                    oUser.createdBy = oParentUser;
                }
                else{
                    console.error("oParentUser is not a Node instance");
                }
            }

            if(oUser.name == "admin"){
                var test = oUser.encodeJSON()
                $.ajax({ dataType: "json", url: "./declarations/userPermissions.json", async: false }).success(function(aData){
                    //console.error(aData);
                    for(var i=0; i<aData.length; ++i){
                        oUser.addPermission(aData[i]["id"], aData[i]["label"]);
                    }
                });
            }

            //Always update admin permissions
            /*$.ajax({ dataType: "json", url: "./declarations/userPermissions.json", async: false }).success(function(aData){
             for(var i=0; i<aData.length; ++i){
             $.extend(true, aData[i], { enabled: true });

             oUser.addPermission(aData[i]["id"], aData[i]);
             }
             });*/

            //Subscribe for model
            $.subscribe("/model/user/"+ strId +"/update/parameter",PubSub.Model.Update.User.Parameter);

            //Subscribe for view
            $.subscribe("/view/user/"+ strId +"/refresh/parameter",PubSub.View.Refresh.User.Parameter);

            //Set parameters if exists
            for(var k in oParam){
                //Skip name, name is set in model["instance"].addUser function
                if(k == "name" || k == "createdBy"){
                    continue;
                }
                if(k == "permissions"){
                    for(var h in oParam[k]){
                        oUser.addPermission(h,oParam[k][h]);
                    }
                    continue;
                }

                //Set parameters
                $.publish("/model/user/"+ strId +"/update/parameter",[oUser,k,oParam[k]]);
            }

            // User interface operations
            if(View.ready){
                View.addUser(oNode);                                    //Add gateway in view
                //$.publish("/view/user/"+ strId +"/refresh/parameter",oUser);
            }

            return oNode;
        }
        catch (e)
        {
            alert(e);
            throw e;
        }


    };

    /** Remove user
     * @param {Node} oNode Node containing gateway to select
     * @returns {Node} New selected node
     */
    this.removeUser = function(oNode){
        var oUser = oNode.getData();
        var strId = oUser.id;
        var oParentUserNode = oUser.createdBy;
        var oConnectedUserNode = model["connectedUser"];

        //TODO Remove children users????
        //Parse all childrens and change createdby attribute
        for(var i=0; i<oUser.childsUsers.length; ++i){
            //console.warn(oUser.childsUsers[i].getData().createdBy);
            oUser.childsUsers[i].getData().createdBy = model["connectedUser"];
        }


        //Remove from model (and check if removed)
        model["instance"].removeUser(oNode);

        if( oUser.name != "admin" ){
            console.error(oParentUser);
            var oParentUser = oParentUserNode.getData();
            //Remove from parent list
            for(var i=0; i<oParentUser["childsUsers"].length; ++i){
                if(oParentUser["childsUsers"][i] == oNode){
                    oParentUser["childsUsers"].splice(i,1);
                    break;
                }
            }
        }

        //Select connected user
        self.selectUser(model["connectedUser"]);


        //Unsubscribe for model
        $.unsubscribe("/model/user/"+ strId +"/update/parameter");          //Unsubscribe update name

        //Unsubscribe for view
        $.unsubscribe("/view/user/"+ strId +"/refresh/parameter");          //Unsubscribe update name

        //Remove from view
        View.removeGateway(oNode);
    };

        this.connectUser = function(oNode){
        if(!(oNode instanceof Node)){
            console.error("Fatal error");
            return;
        }

        model["connectedUser"] = oNode;
        presenter.preferences.setParameter("connectedUser", model["connectedUser"].getData().name);

        self.selectUser(model["connectedUser"]);
    };

    /** Set select user
     * @param {Node} oNode Node containing gateway to select
     * @returns {Node} New selected node
     */
    this.selectUser = function(oNode){
        if(oNode == undefined){
            return;
        }

        var oCurrentNode = model["instance"].getSelectUser();

        //Check selected user
        /*if(oNode === oCurrentNode){
            return;
        }*/

        if(oCurrentNode != undefined){
            var oCurrentUser = oCurrentNode.getData();

            //Save current user
            self.saveUser();
        }

        //Set selected in model
        oNode = model["instance"].setSelectUser(oNode);
        //console.warn(model["instance"].getUIComponentsRoot());

        //Check node
        if(! (oNode instanceof Node) ){
            console.error("oNode is not a Node instance");
            return;
        }

        function emptyTree(oNode){
            if(oNode.children.length > 0){
                for(var i=0; i<oNode.children.length; ++i){
                    emptyTree( oNode.children[i] );
                }
            }

            while(oNode.children.length > 0){
                self.removeUIComponent(oNode.children[0]);
            }

        };

        //Empty interface tree
        emptyTree( model["instance"].getUIComponentsRoot() );

        //Load new user
        self.loadUser(oNode);

        //Set selected in view
        if(View.ready){
            View.selectUser(oNode);
        }

        //console.warn(oNode.encodeJSON());
        return oNode;
    };

    /** Set select user
     * @param {Node} oNode Node containing gateway to select
     * @returns {Node} New selected node
     */
    this.getSelectUser = function(){
        return model["instance"].getSelectUser();
    };

    /** Get User by name
     * @return {User}
     * @param strName
     */

    this.getUserByName = function(strName){

        //Set selected in model
        var oUsersRoot = eveManager.getUsersRoot();

        for(var i=0; i<oUsersRoot.children.length;++i){
            var oNode = oUsersRoot.children[i];
            if(oNode.getData().name == strName){
                return oNode;
            }
        }

        return;
    };

    /** Save user */
    this.saveUser = function(){

        //Get current user
        var oNode = model["instance"].getSelectUser();
        var oUser = oNode.getData();

        //Declare recursive function for decode components
        function uiComponentsSerialize(oNode){

            //oNode.dump("data");
            var oEncode = oNode.encodeJSON();
            //--console.log(oNode);
            //--console.log(oNode.encodeJSON());
            //Push serialization object
            oUser.UI.push(oEncode);

            for(var i=0;i<oNode.children.length;++i){
                uiComponentsSerialize(oNode.children[i]);
            }
        };

        var oInterfaceRoot = eveManager.getUIComponentsRoot();
        oUser.UI = new Array();

        for(var i=0;i<oInterfaceRoot.children.length;++i){
            uiComponentsSerialize(oInterfaceRoot.children[i]);
        }

        //Create project XML file (backward compatibility)
        $.getScript("./parsers/json2xml.js");
    };

    /** Load user
     * @param oNode
     */
    this.loadUser = function(oNode){
        var oUser = oNode.getData();

        //Update users permission
        if(oUser.name == "admin"){
            console.log("Update %cadmin%c permissions","color: blue;","color: inherit;");
            $.ajax({
                url: "./declarations/userPermissions.json",
                cache: false,
                dataType: "json",
                success: function(json){
                    for(var i=0; i<json.length; ++i){
                        if( oUser.addPermission(json[i].id,json[i].label) > 0 ){
                            oUser.setPermission(json[i].id,json[i].label);
                        }
                    }
                },
                error: function(jqXHR,textStatus,errorThrown){
                    console.error(jqXHR);
                }
            });
        }

        //Clear interface
        //model["instance"].resetInterface();

        //Build interface
        for(var i=0; i<oUser.UI.length; ++i){
            var uicomponent = oUser.UI[i];

            var oParent = eveManager.getUIComponentByUid(uicomponent.parentUid);

            //Get maps
            var maps = uicomponent.maps;
            delete uicomponent["maps"];

            var oUIComponentNode = self.addUIComponent(uicomponent,oParent);

            //Add maps
            for(var j=0; j<maps.length; j++){

                //Get MapComponents
                var mapcomponents = maps[j].components;
                delete maps[j]["components"];

                var oMapNode = self.addMapTo(oUIComponentNode,maps[j]);

                //Add MapComponent
                for(var k=0; k<mapcomponents.length; k++){
                    var componentUID = mapcomponents[k].reference;
                    var oComponentReferenceNode = eveManager.getComponentByUid(componentUID);
                    delete mapcomponents[k]["reference"];

                    //Add Component Map
                    self.addComponentToMap(oComponentReferenceNode,oMapNode,mapcomponents[k]);
                }
            }
        }

        //Select first component
        if(oUser.UI[0] != undefined){
            self.selectUIComponent( model["instance"].getUIComponentByUid(oUser.UI[0].UId) );
        }

        presenter.preferences.setParameter("currentUser",oUser.name);
        //presenter.preferences.setParameter("currentUser",oNode);
    };

	/** Gateways */
	
    /** Add gateway
     * @param {Object} oParam Gateway parameters describes by a JSON data type
     * @returns {Node} Node containing new gateway
     */
    this.addGateway = function(oParam){

        //Check UId existance
        var UId = (oParam != undefined)?oParam.UId:undefined;

        //Add gateway to Model
        var oNode = eveManager.addGateway(UId);

        if(! (oNode instanceof Node)){
            console.error("Wrong response from addGateway function Model");
            //$document.trigger("app/load/end");
            return;
        }

        //Add getway to View (call view method???)
        var oGateway = oNode.getData();
        var strId = oGateway.id;

        //Initialize Gateway
        $.loadSyncScript("./components/gateway/init.js");
        onInitGateway(oGateway);

        //Subscribe for model
        $.subscribe("/model/gateways/"+ strId +"/update/name",PubSub.Model.Update.Gateway.Name);            //Subscribe update name
        $.subscribe("/model/gateways/"+ strId +"/update/protocol",PubSub.Model.Update.Gateway.Protocol);    //Subscribe update protocol
        $.subscribe("/model/gateways/"+ strId +"/update/parameter",PubSub.Model.Update.Gateway.Parameter);  //Subscribe update parameter

        //Subscribe for view
        $.subscribe("/view/gateways/"+ strId +"/refresh/name",PubSub.View.Refresh.Gateway.Name);              //Subscribe update name
        $.subscribe("/view/gateways/"+ strId +"/refresh/protocol",PubSub.View.Refresh.Gateway.Protocol);      //Subscribe update protocol
        $.subscribe("/view/gateways/"+ strId +"/refresh/parameter",PubSub.View.Refresh.Gateway.Parameter);    //Subscribe update parameter

        //Set parameters if exists
        if(oParam != undefined){
            for(var k in oParam){
                if(k == "name"){
                    $.publish("/model/gateways/"+ strId +"/update/name",[oGateway,oParam[k]]);
                    //$.publish("/view/gateways/"+ strId +"/refresh/name",oGateway);
                }
                else if(k == "protocol"){
                    $.publish("/model/gateways/"+ strId +"/update/protocol",[oGateway,oParam[k]]);
                    //$.publish("/view/gateways/"+ strId +"/refresh/protocol",oGateway);
                }
                else if(k == "special"){
                    oGateway.addParamVal(k,oParam[k]);
                }
                else{
                    $.publish("/model/gateways/"+ strId +"/update/parameter",[oGateway,k,oParam[k]]);
                    //$.publish("/view/gateways/"+ strId +"/refresh/parameter",oGateway);
                }
            }
        }

        // User interface operations
        if(View.ready){
            var $element = View.addGateway(oNode);                                                          //Add gateway in view
            $.publish("/view/gateways/"+ strId +"/refresh/name",oGateway);
            $.publish("/view/gateways/"+ strId +"/refresh/protocol",oGateway);
            $.publish("/view/gateways/"+ strId +"/refresh/parameter",oGateway);
        }

        return oNode;
    };
    	
    	/** Remove gateway
    	 * @param {Node} oNode Node containing gateway to remove 
         */
    	this.removeGateway = function(oNode){
    	    var oGateway = oNode.getData();
    	    var strId = oGateway.id;
    	    
    	    if(oGateway.isSpecial()){
    	        console.warn("Can not remove %c"+ oGateway.getProtocol() +"%c gateway","color:orange","color:black");
    	        return;
    	    }
    	    
    	    //Remove from model (and check if removed)
    	    eveManager.removeGateway(oNode);
    	    
    	    //Unsubscribe for model
            $.unsubscribe("/model/gateways/"+ strId +"/update/name");          //Unsubscribe update name
            $.unsubscribe("/model/gateways/"+ strId +"/update/protocol");      //Unsubscribe update protocol
            $.unsubscribe("/model/gateways/"+ strId +"/update/parameter");     //Unsubscribe update parameter
            
            //Unsubscribe for view
            $.unsubscribe("/view/gateways/"+ strId +"/refresh/name");          //Unsubscribe update name
            $.unsubscribe("/view/gateways/"+ strId +"/refresh/protocol");      //Unsubscribe update protocol
            $.unsubscribe("/view/gateways/"+ strId +"/refresh/parameter");     //Unsubscribe update parameter
            
            //Remove from view
            View.removeGateway(oNode);
            
            //Select ghost gateway by default (getGatewaysByProtocol function returns an array)
            self.selectGateway(self.getGatewaysByProtocol("ghost")[0]);
    	};
    	
    	/** Set select gateway
         * @param {Node} oNode Node containing gateway to select
         * @returns {Node} New selected node
         */
    	this.selectGateway = function(oNode){
    	    if(oNode == undefined){
    	        oNode = eveManager.getDefaultGateway(); 
    	    }
    	    
    	    //Set selected in model
    	    oNode = eveManager.setSelectGateway(oNode);
    	    
    	    //Check node
    	    if(! (oNode instanceof Node) ){
    	        console.error("oNode is not a Node instance");
    	        return;
    	    }
    	    
    	    //Set selected in view
    	    View.selectGateway(oNode);
    	};
    	
    	/** Get selected gateway
         * @returns {Node} Selected node
         */
        this.getSelectGateway = function(){
            return eveManager.getSelectGateway();
        };
    	
    	/** Get Gateway by protocol
    	 * @param {string} strProtocol String containing protocol Id (check protocol Id on "./declarations/protocols.json") 
         * @return {Array} Array containing all gateway's node with specified protocol
         */
        this.getGatewaysByProtocol = function(strProtocol){
            //$document.trigger("app/load/begin",["Find gateways"]);
            
            //Set selected in model
            var oGatewayRoot = eveManager.getGatewaysRoot();
            var aGateways = new Array();
            
            for(var i=0; i<oGatewayRoot.children.length;++i){
                var oNode = oGatewayRoot.children[i];
                if(oNode.getData().getProtocol() == strProtocol){
                    aGateways.push(oNode);
                }
            }
            
            //$document.trigger("app/load/end");
            return aGateways;
        };

        this.getGatewayByUid = function(UId){
            return model["instance"].getGatewayByUid(UId);
        };
	
	/** Component */

        /** Add component
         * @param {Object|Node} oParam JSON data type or Node describing component to add
         * @param {Node} oParent Parent node to add new child component node
         * @returns {Node} Node containing new UI component
         */
        this.addComponent = function(oParam,oParent,bImport){
            //Check input parameters
            if(typeof oParam == "string"){
                var type = oParam;
                oParam = { type: type };
            }
            
            if(oParam == undefined && oParent == undefined){
                oParam = {};
                console.error("Component type required.");
                //$document.trigger("app/load/end");
                return undefined;
            }
            else if(oParam == undefined){
                oParam = {};
                console.error("Component type required.");
                //$document.trigger("app/load/end");
                return undefined;
            }
            else if(oParam instanceof Node){
                oParent = oParam;
                oParam = {};
                console.error("Component type required.");
                //$document.trigger("app/load/end");
                return undefined;
            }
            
            if(oParent == undefined) {

                oParent = eveManager.getSelectComponent();
                if (oParent!=undefined) {
                    if (oParent.data.type != "group") {
                            oParent = eveManager.getComponentByUid(oParent.data.parentUId);
                    }
                    else
                    {

                        try{
                            if ($("#" + application.getSelectComponent().getData().id).accordion("instance").active.length == 0) {
                                oParent = eveManager.getComponentByUid(oParent.data.parentUId);

                            }

                        }
                        catch (e)
                        {
                            oParent = eveManager.getComponentsRoot();
                        }
                    }



                }
                if(oParent == undefined) {
                    oParent = eveManager.getComponentsRoot();
                }
            }

            //Get node from Model
            var oNode = model["instance"].addComponent(oParam.UId,oParent);

            if(!(oNode instanceof Node)){
                console.error("Wrong response from addComponent Model function");
                //$document.trigger("app/load/end");
                return undefined;
            }
            
            //Initialize Component
            var oComponent = oNode.getData();
            var strId = oComponent.id;
            var strType = oParam.type;
            
            //Check external functions
            //var instanceOnDefine = window.onDefineComponent;
            //var instanceOnInit = window.onInitComponent;
            //var instanceOnUpdate = window.onUpdateComponent;
            delete window.onDefineComponent;
            delete window.onInitComponent;
            //delete window.onUpdateComponent;
            
            //Initialize component
            //$.loadSyncScript("./components/"+ strType +"/define.js");
            //$.loadSyncScript("./components/"+ strType +"/init.js");
            //$.loadSyncScript("./components/"+ strType +"/update.js");

            eval( QT.readFile("./components/"+ strType +"/define.js") );
            eval( QT.readFile("./components/"+ strType +"/init.js") );
           
            //Component initialization
            oComponent.setType(strType);
            oComponent.parentUId = oParent.getData().UId;
            try {
                onDefineComponent(oComponent);
                onInitComponent(oComponent);
            }
            catch(error){
                // capita quando viene caricato un progetto vecchio ed un componente non esiste più nella lista di quelli attuali
                return undefined;

            }



            //Subscribe for model
            $.subscribe("/model/components/"+ strId +"/update/parameter",PubSub.Model.Update.Component.Parameter);
            
            //Subscribe for view
            $.subscribe("/view/components/"+ strId +"/refresh/parameter",PubSub.View.Refresh.Component.Parameter);
            $.subscribe("/view/component/"+ strId +"/refresh/references",PubSub.View.Refresh.Component.References);

            for(var k in oParam){
                if(k != "type"){
                    if(k.toLowerCase() == "special"){
                        oComponent.addParamVal(k,oParam[k]);
                        continue;
                    }
                    else if(k.toLowerCase() == "gateway"){
                        var oGatewayNode;

                        if(oParam[k] instanceof Node){
                            oGatewayNode = oParam[k];
                        }
                        else{
                            oGatewayNode = eveManager.getGatewayByUid(oParam[k]);
                        }

                        $.publish("/model/components/"+ strId +"/update/parameter",[oComponent,k,oGatewayNode]);
                        //oComponent.UId = oParam[k];
                    }
                    else if(k.toLowerCase() == "descriptor"){
                        var oDescriptor = oComponent.getDescriptor();
                        oDescriptor.setValueDescription(oParam[k].type,oParam[k].min,oParam[k].max,oParam[k].value);
                        oDescriptor.setEditable(oParam[k].editable);
                        oDescriptor.setDefaultAction(oParam[k].action);
                        var oDescriptions = oParam[k].descriptions;
                        for(var h in oDescriptions){
                            oDescriptor.addDescription(h,oDescriptions[h]);
                        }
                    }
                    else if(k.toLowerCase() == "icons"){
                        // comment this part during download this part to keep the default icon instead of the project one
                        //for(var i=0; i<oParam[k].length; ++i){

                        if (oParam[k][0].srcId=="app/resource/0")
                        {
                            oParam[k][0].srcId="EVEIcons";
                        }
                        for(var i=0; i<oParam[k].length; ++i){
                             //   oParam[k][i] = $.extend(true, {}, (new Icon()), oParam[k][i]);
                            oComponent.m_Data.icons[i]= $.extend(true, {}, (new Icon()), oParam[k][i]);


                        }
                        
                        //oComponent.setParamVal(k,oParam[k]);

                        // until here
                        continue;
                    }
                    else{
                        if (k!="viewmodelist")
                            $.publish("/model/components/"+ strId +"/update/parameter",[oComponent,k,oParam[k]]);
                        //$.publish("/view/components/"+ strId +"/refresh/parameter",[oComponent,k]);
                    }
                }
            }


            // User interface operations 
            if(View.ready){
                var $openGroup=$("#"+ oParent.getData().id+" >ul").css("display");
                if ($openGroup!="none" || oParent==eveManager.getComponentsRoot())
                {
                    var $element = View.addComponent(oNode,oParent);                                                //Add gateway in view
                    // set a pointer to the view HTML to speedup operations later
                    oComponent.HTMLViewReference=$element;
                    $.publish("/view/components/"+ strId +"/refresh/parameter",[oComponent,"name"]);
                    $.publish("/view/components/"+ strId +"/refresh/parameter",[oComponent,"icons"]);

                }
            }

            //Update dependencies
         //   $.publish("/view/components/refresh/dependencies",[bImport]);
            return oNode;
        };
        
        /** Remove component
         * @param {Node} oNode Node containing component to remove
         */
        this.removeComponent = function(oNode){
            if (oNode==null) return;
            var oComponent = oNode.getData();
            var strId = oComponent.id;
            //console.log("Remove component : "+strId);
            if(oComponent.getParamVal("special") != undefined){
                console.warn("Can not remove "+ oComponent.getType());
                return;
            }

            //
            //console.log(oNode);
            if (oNode.getData().type=="weektimer")
            {

                if (oNode.getData().getParamVal("gui/default")==true)
                {
                    var timers=application.getComponentsByType("weektimer");
                    //    console.log(timers);
                    //    console.log(strParam);

                    // application.getComponentsByType("weektimer")[0].getData().setParamVal("gui/default",false)
                    for (var k in timers)
                    {
                        if (timers[k]!=oNode)
                        {
                            timers[k].getData().setParamVal("gui/default",true)
                            break;

                        }
                    }

                }
            }

            if (oNode.getData().type=="weektimercalendar")
            {

                if (oNode.getData().getParamVal("gui/default")==true)
                {
                    var timers=application.getComponentsByType("weektimercalendar");
                    //    console.log(timers);
                    //    console.log(strParam);

                    // application.getComponentsByType("weektimer")[0].getData().setParamVal("gui/default",false)
                    for (var k in timers)
                    {
                        if (timers[k]!=oNode)
                        {
                            timers[k].getData().setParamVal("gui/default",true)
                            break;

                        }
                    }

                }
            }

            
            //Before remove component need to remove all component interface reference
            var aRefComponents = oComponent.getParamVal("interfacesLinkedUIComponents");

            if(aRefComponents != undefined){
                var oCorrect = true;
                while(oCorrect && aRefComponents.length > 0){
                    var oDataNode = aRefComponents.shift();
                    var oData = oDataNode.getData();
                    
                    //Remove UICompnent
                    if(oData instanceof UIComponent){
                        if(!self.removeUIComponent(oDataNode)){
                            console.error("Can not remove reference UIComponent");
                            oCorrect = false;
                        }
                    }
                    
                    //Remove MapComponent
                    else if(oData instanceof MapComponent){
                        if(!self.removeMapComponentFrom(oDataNode)){
                            console.error("Can not remove reference MapComponent");
                            oCorrect = false;
                        }
                    }
                    
                    //Unknown type of component
                    else{
                        oCorrect = false;
                        console.error("Unknown type of javascript object");
                    }
                }
            }

            //Remove all child components
            while(oNode.children.length > 0){
                self.removeComponent(oNode.children[0]);
            }
            //TODO Verificare se è il caso di eliminare anche le mappe
            
            //Remove from model (and check if removed)
            //console.warn($("#componentscontainer").data("eveManagerNode").dump(["data","m_Data","name"]));
            var bCheck = eveManager.removeComponent(oNode);

            if(!bCheck){
                console.error("Can not remove component");
                return;
            }
            
            //console.warn($("#componentscontainer").data("eveManagerNode").dump(["data","m_Data","name"]));
            
            //Unsubscribe for model
            $.unsubscribe("/model/components/"+ strId +"/update/parameter");
            
            //Unsubscribe for view
            $.unsubscribe("/view/components/"+ strId +"/refresh/parameter");
            $.unsubscribe("/view/component/"+ strId +"/refresh/references");
            
            //Remove from view
            View.removeComponent(oNode);

            oNode = undefined;
            
            //Update dependencies
            $.publish("/view/components/refresh/dependencies");
        };
        
        /** Set select component
         * @param {Node} oNode Node containing component to select
         * @returns {Node} New selected node
         */

        this.fillComponentsInAccordion= function(oNode,$element){
                for (var i = 0; i < oNode.children.length; ++i) {
                    var oComponent = oNode.children[i];
                    var strId = oComponent.getData().id;
                    View.addComponent(oComponent, oNode, $element);
                    try{
                        $.publish("/view/components/" + strId + "/refresh/parameter", [oComponent.getData(), "name"]);
                        $.publish("/view/components/" + strId + "/refresh/parameter", [oComponent.getData(), "icons"]);

                    }
                    catch(ex)
                    {

                    }

                }


                if (typeof onGroupOpen!="undefined")
                    onGroupOpen();
                if ($("#tabs").tabs("instance").options.active==1) //components
                {
                    try{$("#componentscontainer li").draggable("destroy");}catch(ex){};

                    // se la finestra di scelta componenti  è aperta...
                    if ($("#addComponentsList").parent().attr("style").indexOf("block")>0)
                    {
                        $("#componentscontainer li").draggable({
                            appendTo: "body",
                            containment: "body",
                            revert: "invalid",
                            revertDuration: false,
                            connectToSortable: ".actions",
                            //helper: "clone",
                            helper: function(e){ return $(e.currentTarget).clone(true); },
                            zIndex: 1000
                        });
                    }
                    // se il componente selezionato è uno scenario (con il blocco nella scelta componente...
                    if (typeof onGroupOpen!="undefined")
                        onGroupOpen();



                }
                else
                {
                    $("#componentscontainer li").draggable({
                        appendTo: "body",
                        connectToSortable: "  #mylists .level",
                        helper: function (e) {
                            return $(e.currentTarget).clone(true);
                        },
                        /*
                         revert: "invalid",
                         revertDuration: false,

                         //helper: "clone",

                         */zIndex: 1000
                    });

                }
                if ($("#tabs").tabs("instance").options.active==2) // classic
                {
                    $("#componentscontainer").find(".eve-managerui-onlyMap").hide();
                }
                else
                {
                    $("#componentscontainer").find(".eve-managerui-onlyMap").show();
                }

          /*    var strConnectWith = "#mycomponents,#componentscontainer,#myinterface .level,#mylists .level, #interface-maps .maps-container .viewer";

                var $noGroupComponents = $element.find("[eve-manager-component]:not([eve-manager-component='group'])");
                var strInterfaceElementSelector = "#mycomponents ,#lists-components li, #interface-elements li, #interface-components li";
                $(strInterfaceElementSelector).draggable({
                    appendTo: "body",
                    containment: "body",
                    revert: "invalid",
                    revertDuration: false,
                    connectToSortable: strConnectWith,
                    //helper: "clone",
                    helper: function(e){ return $(e.currentTarget).clone(true); },
                    zIndex: 1000
                });*/
        };

        this.deleteComponentsInAccordion= function(oNode,$element){
            //console.log (" in chiusura");


            function clearHTMLInside(oNode) {
                var oData = oNode.getData();
                oData.HTMLViewReference=null;
                for(var i=0;i<oNode.children.length;++i){
                    clearHTMLInside(oNode.children[i]);
                }
            }
            var oComponentRoot = eveManager.getComponentsRoot();

            try{

                for(var i=0;i<oNode.children.length;++i){

                    clearHTMLInside(oNode.children[i]);
                }
            }
            catch(e){};

            $element.find(">ul").empty();

        };

        this.selectComponent = function(oNode,$element){
        if(oNode == undefined){
            return;
        }

        if (typeof onDeselect!="undefined")
            if (onDeselect()==false)
                return;

            if ($element!=undefined)
            {
                if(eveManager.getSelectComponent() == oNode){
                    return oNode;
                }

            }

   //         $.publish("/view/components/refresh/dependencies",false);
            if(oNode.getParent() == undefined){
                console.warn("This component don't exist anymore");
                return;
            }



            //console.log("Selected component " + oNode);
            self.deselectComponent(oNode);                                  //Deselect current component
            oNode = eveManager.setSelectComponent(oNode);                   //Set selected in model

            //Check node
            if(! (oNode instanceof Node) ){
                console.error("oNode is not a Node instance");
                return;
            }

            //Get node properties
            //var oComponent = oNode.getData();
            //var strType = oComponent.getType();

            //Set selected in view
            View.selectComponent(oNode);
            if (typeof onSelect!="undefined")
                onSelect();
            return oNode;
        };
        
        /** Get selected component         
         * @returns {Node} Selected node
         */
        this.getSelectComponent = function(){
            return eveManager.getSelectComponent();
        };
        
        /** Delect component
         */
        this.deselectComponent = function(){
            //Set selected in model
            eveManager.deselectComponent();
            
            //Deselected in view
            View.deselectComponent();
        };
	    
	    /** Check if exist a component with same parameters
         * 
         */
        this.checkComponentExist = function(oParam,oRoot){
            //Check
            if(oParam == undefined){
                return false;
            }
            if(oRoot == undefined){
                oRoot = model.instance.getComponentsRoot();
            }
            
            //Parse parameters
            var find = false;
            var oComponent = oRoot.getData();
            
            //if(oComponent.getParamVal != undefined)
                //if(oComponent.getParamVal("name") == oParam["name"])
                //console.log("%c"+ oComponent.getParamVal("name"),"color:grey;");
            
            if(oComponent.encodeJSON != undefined){
                var oSerialized = oComponent.encodeJSON();
                find = true;
                
                //Check parameters
                for(var k in oParam){

                    //Filter params
                    /* if(k == "name" || k == "icons" || k == "values" || k == "Ui"){ */
                    if(
                        k == "name"     ||
                        k == "icons"    ||
                        k == "UId"      ||
                        k == "values"
                    ){
                        continue;
                    }

                    var sParameter = oParam[k];

                    //Get Gateway ID
                    if(k == "gateway"){
                        sParameter = oParam[k].encodeJSON().UId;
                    }

                    if(sParameter != oSerialized[k]){

                        /*if(oParam[k] == "combo"){
                            if(oParam["name"] == oSerialized["name"]){
                                //console.error(k);
                                console.error(oParam[k]);
                                console.error(oSerialized[k]);
                            }
                        }*/
                    
                        find = false;
                        break;
                        //continue;
                    }
                    
                    /*if(oParam[k] == "combo"){
                        if(oParam["name"] == oSerialized["name"]){
                            if(oParam["type"] == oSerialized["type"]){
                                //console.error(k);
                                console.log("%c"+oParam[k],"color:green;");
                                console.log("%c"+oSerialized[k],"color:green;");
                                console.log(oParam);
                                console.log(oSerialized);
                            }
                        }
                    }*/
                    
                    //Success
                    find = oComponent;
                }
            }
            
            if(find){
                //return oComponent;
                return oRoot;
            }
            else{
                for(var i=0; i<oRoot.children.length; ++i){
                    
                    find = self.checkComponentExist(oParam,oRoot.children[i]);
                    if(find != false){
                        break;
                    }
                }
                return find;
            } 
        };

        /** Get component by UId
         *
         */
        this.getComponentByUid = function(sId,oRoot){
            return model.instance.getComponentByUid(sId,oRoot);
        };
        
         /** Get component by Name
         *
         */
        this.getUIComponentByName = function(sId,oRoot){      	
            return model.instance.getUIComponentByName(sId,oRoot);
        };
        
         /** Get component by UserData
         *
         */
        this.getUIComponentByUserData= function(sId,oRoot){      	
            return model.instance.getUIComponentByUserData(sId,oRoot);
        };

	    /** Get components by type
         * 
         */
        this.getComponentsByType = function(strType){
            var oComponentsRoot = model["instance"].getComponentsRoot();
            var aComponents = new Array();
            var findByType = function(sType,oNode){
                var oComponent = oNode.getData();
                
                
                if(oComponent.getType != undefined && oComponent.getType() == sType){
                    aComponents.push(oNode);
                }

                if(oComponent.getType == "*"){
                    aComponents.push(oNode);
                }


                for(var i=0;i<oNode.children.length;++i){
                    findByType(sType,oNode.children[i]);
                }
                    
            };

            var temp = new Array();
            temp = strType.split(",");
            for (a in temp ) {
                findByType(temp[a], oComponentsRoot);
            }
            return aComponents;
        };

    /** Get components by tags
     *
     */
    this.getComponentsByTags = function(strTag){
        var oComponentsRoot = model["instance"].getComponentsRoot();
        var aComponents = new Array();
        var findByTag = function(strTag,oNode){
            var oComponent = oNode.getData();


            if(oComponent.getTag != undefined &&  oComponent.includeTag(strTag)){
                aComponents.push(oNode);
            }

            for(var i=0;i<oNode.children.length;++i){
                findByTag(strTag,oNode.children[i]);
            }

        };

        var temp = new Array();
        temp = strTag.split(",");
        for (a in temp ) {
            findByTag(temp[a], oComponentsRoot);
        }
        return aComponents;
    };

    /** UI Component */
	
	   	/** Add UI component
         * @param {Object|Node} oParam JSON data type or Node describing UI component to add
         * @param {Node} oParent Parent node to add new child UI component node
         * @param {String} UILevel Visual position of the icon in the classic interface levels
         * @returns {Node} Node containing new UI component 
         */
        this.addUIComponent = function(oParam,oParent,UILevel){
            
            //Check input parameters (oParam could be a string, an object, a Node)
            
            // oParam is undefined
            console.log("pinocchio");
            if(oParam == undefined){
                
                var oDefaultPan = self.ResoucesManager.getDefaultIconsPan();
                
                //Default icon
                oParam = {
                    icon: $.extend(true,(new Icon()),{
                        position:{ x:0, y:0 },
                        //srcId: oDefaultPan["params"]["src"]
                        srcId: oDefaultPan["id"],
                        size: { h: oDefaultPan["params"]["iconsHeight"], w: oDefaultPan["params"]["iconsWidth"] },
                        setSize: { h: oDefaultPan["params"]["imgHeight"], w: oDefaultPan["params"]["imgWidth"] }
                    }) 
                };
                
            }
            // oParam is a string (name of icon)
            else if(typeof oParam == "string"){
                /*if(oParam != "icon"){
                    console.error("Can't insert as input a string Type different by \"icon\"");
                    return;
                }*/
                var iconDescription = oParam;
                 
                //var type = oParam;
                oParam = {
                    //type: type
                    //type: "icon"
                };
            }
            // oParam is a Component reference Node
            else if(oParam instanceof Node){
                if(! (oParam.getData() instanceof Component) ){
                    console.error("Only a node containing Component instance can be added to interface");
                    return;
                }
                
                var oNode = oParam;
                var oComponent = oNode.getData();

                var oIcon = oComponent.getParamVal("icons")[0];
                
                /*########  CREATE A JSON OBJECT FROM NODE  ########*/
               
               
                /*########  OR CREATE A REFERENCE FROM COMPONENT TO UICOMPONENT
                 * (probably not useful, get Component Uid from here and set Component reference after UIComponent creation
                 *  so an UId is created for sure)   ########*/
                //console.error( oComponent.getParamVal("interfacesLinks").push() );
                
                /*
                oRefNode = oParam;
                var oRefComponent = oParam.getData();
                
                if(!oRefComponent instanceof Component){
                    console.error("Only an input Node that contains an instance of Component as data field is accepted");
                    return;
                }
                else if(oRefComponent.getType() == "group"){
                    console.error("Can't use group component as interface component");
                    return;
                }*/
                
                oParam = {
                    // Set Component UI reference
                    referenceUId: oComponent.UId,
                    
                    icon: oIcon
                    //type: (oRefComponent.getType() != "scenario")?"component":"scenario"
                };
              
            }
            // oParam is a JSON object
            else if(typeof oParam == "object"){
            }
            
            // Parent check
            if(oParent == undefined){
                oParent = eveManager.getUIComponentsRoot();
            }
            /*else if(! (oParent.getData() instanceof UIComponent) ){
                console.error("Can not append component in a parent different by UIComponent");
                return;
            }*/
            
            //Get node from Model (oParam.UId could be undefined)
            if (UILevel==undefined ) UILevel=oParam.UILevel;
            var oNode = eveManager.addUIComponent(oParam.UId,oParent,UILevel);
            
            //Check model response
            if(! (oNode instanceof Node) ){
                console.error("Wrong response from addUIComponent Model function");
                return undefined;
            }
            
            //Initialize Component
            var oUIComponent = oNode.getData();
            var strId = oUIComponent.id;
            var strType = oParam.type;
            
            oUIComponent.parentUId = oParent.getData().UId;                                   //
            oUIComponent.referenceUId = oParam.referenceUId;                                  //
            
            //Set UIComponent parameters (eg. name)
            var oRefNode;
            var oRefComponent;
            
            oUIComponent.setParamVal("name",self.Dictionary.getName("Untitled"));

            if(oUIComponent.referenceUId != undefined){
                //Append this UIComponent node to Component reference

                oRefNode = eveManager.getComponentByUid(oUIComponent.referenceUId);
                if (oRefNode != undefined)
                {

                    oRefComponent = oRefNode.getData();
                    oRefComponent.getParamVal("interfacesLinkedUIComponents").push(oNode);
                    oUIComponent.setParamVal("name",oRefComponent.getParamVal("name"));
                }
                else
                {
                    console.log("Component undefined");
                    console.log(oUIComponent.referenceUId);
                    console.log(oRefNode);
                    return;
                }
            }
            
            //oUIComponent.setType(strType);                                                  //Set type to User Interface Component
            //oUIComponent.setType("icon");                                                  //Set type to User Interface Component
            //oUIComponent.setRefComponent(oRefNode);                                         //Set node as reference component
            
            //console.debug("%cCall self.addMap methods and set name or initialize map","color:orange");
            /*var oIconData = {
                position: { x: 0, y: 0 },
                //color: { R:255, G:255, B:255 },
                color: { R:255, G:0, B:0 },
                colorHex:"#FFFFFF",
                type: undefined
            };*/
            
            //Subscribe for model
            $.subscribe("/model/uicomponents/"+ strId +"/update/parameter",PubSub.Model.Update.UIComponent.Parameter);
            $.subscribe("/model/uicomponents/"+ strId +"/update/map",PubSub.Model.Update.UIComponent.Map);
            
            //Subscribe for view
            $.subscribe("/view/uicomponents/"+ strId +"/refresh/parameter",PubSub.View.Refresh.UIComponent.Parameter);
            $.subscribe("/view/uicomponents/"+ strId +"/refresh/map",PubSub.View.Refresh.Maps);     //Called by View UIComponent selection
            
            //Set parameters if exists
            for(var k in oParam){
                //Jump for type parameter
                if(k != "type"){
                    if(k == "special"){
                        oUIComponent.addParamVal(k,oParam[k]);
                        continue;
                    }
                    else if(k == "UId"){
                        //console.warn(oUIComponent.UId +" - "+ oParam[k]);
                        continue;
                    }
                    else if(k == "icon"){
                        // Merge icon properties (default properties with new properties)
                        var oIcon = oUIComponent.getParamVal("icon"); 
                        var oNewIcon = $.extend(true,{
                            
                        },oIcon,oParam[k]);
                        
                        oUIComponent.setParamVal("icon",oNewIcon);
                        continue;
                    }
                    else if(k == "maps"){
                        for(var i=0; i<oParam[k].length; ++i){
                            var oMap = oParam[k][i];
                            for(var j=0; j<oMap.components.length; ++j){
                                var nRefUId = oMap.components[j].reference;
                                oMap.components[j].reference = eveManager.getComponentByUid(nRefUId);
                            }
                            self.addMapTo(oNode,oMap);
                        }
                        continue;
                    }
                    else if(k == "referenceUId"){
                        oUIComponent.referenceUId = oParam[k];
                        continue;
                    }
                    else if(k == "parentUid"){
                        oUIComponent.parentUId = oParam[k];
                        continue;
                    }
                    else if(k == "UILevel"){
                        oUIComponent.UILevel = oParam[k];
                        continue;
                    }
                    else{
                        oUIComponent.setParamVal(k,oParam[k]);
                        //$.publish("/model/uicomponents/"+ strId +"/update/parameter",[oUIComponent,k,oParam[k]]);
                        continue;
                    }
                }
            }
            
            // User interface operations 
            if(View.ready){
                var $element = View.addUIComponent(oNode,oParent);                  //Add UI component in view
                $.publish("/view/uicomponents/"+ strId +"/refresh/parameter",oUIComponent,"name");
                //$.publish("/view/uicomponents/"+ strId +"/refresh/parameter",oUIComponent,"icon");
            }
            
            //Update dependencies
         //   $.publish("/view/uicomponents/refresh/dependencies");
            return oNode;
        };
	    
	    /** Remove UI component
         * @param {Node} oNode Node containing UI component to remove
         */
        this.removeUIComponent = function(oNode){
            var oUIComponent = oNode.getData();
            var strId = oUIComponent.id;
            
            self.deselectUIComponent();
            
            //Extract UIComponent from Component reference array
            if(oUIComponent.referenceUId != undefined){
                oRefNode = eveManager.getComponentByUid(oUIComponent.referenceUId);
                if (oRefNode != undefined)
                {
                    var oComponent = eveManager.getComponentByUid(oUIComponent.referenceUId).getData();
                    var aUIComponents = oComponent.getParamVal("interfacesLinkedUIComponents");
                    for(var i=0; i<aUIComponents.length; ++i){
                        if(oUIComponent.UId == aUIComponents[i].getData().UId){
                            aUIComponents.splice(i,1);
                            break;
                        }
                    }
                }
            }

            //Remove all child components
            while(oNode.children.length > 0){
                self.removeUIComponent(oNode.children[0]);
            }
            
            //Get node Parent
            var oNodeParent = oNode.getParent();
            
            //Remove from model (and check if removed)
            if(eveManager.removeUIComponent(oNode) != 0){
                console.error("Can not remove component");
                return;
            }
            
            //Remove Maps
            var aMaps = oUIComponent.getMaps().children;
            var oCorrect = true;
            while(oCorrect && aMaps.length > 0){
                if(!self.removeMapFrom(aMaps[0],oNode)){
                    oCorrect = false;
                }
            }
            
            //Unsubscribe for model
            $.unsubscribe("/model/uicomponents/"+ strId +"/update/parameter");
            $.unsubscribe("/model/uicomponents/"+ strId +"/update/map");
            
            //Unsubscribe for view
            $.unsubscribe("/view/uicomponents/"+ strId +"/refresh/parameter");
            $.unsubscribe("/view/uicomponents/"+ strId +"/refresh/map");
            
            //Remove from view
            View.deselectUIComponent(oNode);
            View.removeUIComponent(oNode);
            
            //oNode = undefined;
            
            //Update dependencies
            $.publish("/view/uicomponents/refresh/dependencies");
            
            //Select first UIComponent
            var oFirstNode = oNodeParent.children[0];
            
            if(oFirstNode != undefined){
                self.selectUIComponent(oFirstNode);
            }
           
           return true;
        };
	    
	    /** Set select UI component
         * @param {Node} oNode Node containing UI component to select
         * @returns {Node} Node containing selected UI component
         */
	    this.selectUIComponent = function(oNode){
	        if(! (oNode instanceof Node) ){
	            console.error("oNode is not a Node instance");
	            return false;
	        }
	        
	        /*if(oNode.getParent() == undefined){
                console.warn("This component don't exist anymore");
                return;
            }*/

            self.deselectUIComponent(oNode);                                     //Deselect current component
            
            oNode = eveManager.setSelectUIComponent(oNode);                 //Set selected in model
            
            //Check node
            if(! (oNode instanceof Node) ){
                console.error("oNode is not a Node instance");
                return false;
            }
            
            //Load html component file
            var oUIComponent = oNode.getData();

            //Maps mode
            if(oUIComponent.getMaps().children.length < 1){
                console.debug("Add default map");
                self.addMapTo(oNode);                                       //Add default map
            }

            //console.error(oUIComponent.encodeJSON());
            
            //Set selected in view
            if(View.ready){
                View.selectUIComponent(oNode);
            }
            
            //Get first map node
            var oFirstMapNode = oUIComponent.getMaps().children[0];
            if(oFirstMapNode == undefined){
                console.error("Any map for component: "+ oUIComponent.toString());
                return;
            }
            
            //Select first map node
           self.selectMap(oFirstMapNode);
            
            return oNode;
	    };
	    
	    /** Get selected UI component         
         * @returns {Node} Selected node
         */
        this.getSelectUIComponent = function(){
            return eveManager.getSelectUIComponent();
        };
	    
	    /** Deselect UI component
         */
        this.deselectUIComponent = function(NewONode){
            //Deselect node
            try
            {
                var oNode = eveManager.deselectUIComponent();
                console.log(NewONode);
                var newLevel=NewONode.getData().UILevel;
                var oldLevel=oNode.getData().UILevel;
                console.log(newLevel);
                console.log(oldLevel);
                $("#"+newLevel+" .select").removeClass("select");
                if (NewONode.getData().UILevel==oNode.getData().UILevel)
                {
                    View.deselectUIComponent(oNode);
                }    //Deselected in view


            }
            catch (e) {}
        };

    /**
     *
     * @param UId
     * @returns {*|Node|undefined}
     */
    this.getUIComponentByUid = function(UId){
        return model["instance"].getUIComponentByUid(UId);
    }
        
    /** Maps (this function don't have its corresponding function in Model or Viewer) */
        
    /** Add Map to Component UI Component
     *
     */
    this.addMapTo = function(oNode,oParam){
        //Check input oNode existance
        if(oNode == undefined){
            console.error("Specify an oNode to add map");
            return;
        }

        //Check if oNode contain UIComponent
        if(! (oNode.getData() instanceof UIComponent) ){
            console.error("Map can add only to a UIComponent instance");
            return;
        }

        //Check input parameters
        var oDefaultPan = application.ResoucesManager.getDefaultIconsPan();



        var panw=model["instance"].skin.IconsPan[0].imgWidth;
        var panh=model["instance"].skin.IconsPan[0].imgHeight;
        var iconw=model["instance"].skin.IconsPan[0].iconsWidth;
        var iconh=model["instance"].skin.IconsPan[0].iconsHeight;

        var i0=model["instance"].skin.ComponentsInfo.MapTab.defaultIcons[0];



        var oDefaultParam = {
            name: _D("Untitled"),
            icon: $.extend(true, (new Icon()), {
                position:{  x: -i0.position.x*iconw, y: -i0.position.y*iconh },
                enableColors: false,
                srcId: oDefaultPan["id"],
                size: { h: iconh, w: iconw  },
                setSize: {  h: panh, w: panw  }
            })
        };

        //Set default parameters
        if(oParam == undefined){
            oParam = oDefaultParam;
        }
        else{
            $.extend(true, oDefaultParam, oParam);
        }

        //Create component in model
        var oUIComponent = oNode.getData();
        var strId = oUIComponent.id;                                                  //Get id
        var oMapNode = oUIComponent.addMap();                                         //Add map
        var oMap = oMapNode.getData();                                                //Get map object
        oMap.name = oParam.name;                                                      //Set name

        //Subscribe for Model
        $.subscribe("/model/uicomponent/map/"+ oMap.id +"/update/parameter",PubSub.Model.Update.Map.Parameter);

        //Subscribe for View
        $.subscribe("/view/uicomponent/map/"+ oMap.id +"/refresh/parameter",PubSub.View.Refresh.Map.Parameter);

        for(var k in oParam){
            //Set input parameters
            $.publish("/model/uicomponent/map/"+ oMap.id +"/update/parameter",[oMap,k,oParam[k]]);
            //$.publish("/view/uicomponent/map/"+ oMap.id +"/refresh/parameter",[oMap,k]);
        }

        // User interface operations
        if(View.ready && oNode === self.getSelectUIComponent()){
            View.addMap(oMapNode);
            $.publish("/view/uicomponent/map/"+ oMap.id +"/refresh/parameter",oMap,"name");
        }

        //Choose if return Map node or just Map object
        return oMapNode;
        //return oMap;
    };
        
    /** Remove Map from Component UI Component
     *
     */
    this.removeMapFrom = function(oNode,oNodeUIComponent){
        //Check input oNode existance
        if(oNode == undefined){
            console.error("Specify an oNode to remove map");
            return;
        }

        //Check if oNode contain UIComponent
        if(! (oNode.getData() instanceof Map) ){
            console.error("oNode not contains Map instance");
            return;
        }

        //Check input oNode existance
        if(oNodeUIComponent == undefined){
            console.error("Specify an oNodeUIComponent to remove map");
            return;
        }

        //Check if oNode contain UIComponent
        if(! (oNodeUIComponent.getData() instanceof UIComponent) ){
            console.error("oNodeUIComponent not contains UIComponent instance");
            return;
        }

        var oUIComponent = oNodeUIComponent.getData();
        var oMap = oNode.getData();

        //Remove map from model
        oUIComponent.removeMap(oNode);

        //Remove MapComponents
        var oCorrect = true;
        while(oCorrect && oMap.components.length > 0){
            if(!self.removeMapComponentFrom(oMap.components[0],oNode)){
                oCorrect = false;
            }
        }

        //Unubscribe for Model
        $.unsubscribe("/model/uicomponent/map/"+ oMap.id +"/update/parameter");

        //Unubscribe for View
        $.unsubscribe("/view/uicomponent/map/"+ oMap.id +"/refresh/parameter");

        if(View.ready){
            View.removeMap(oNode);
        }

        //Select automatically first map
        var oFirstMap = oUIComponent.getMaps().children[0];
        if(oFirstMap == undefined){
            return;
        }
        self.selectMap(oFirstMap);

        return true;
    };

    /** Select Map
     *
     */
    this.selectMap = function(oMapNode){
        var oNode = self.getSelectUIComponent();

        if(oNode == undefined){
            return;
        }

        var oUIComponent = oNode.getData();

        self.deselectMap();                                             //Deselect current map

        oMapNode = oUIComponent.setSelectMap(oMapNode);                 //Select map


        if(oMapNode == undefined){
            console.error("Any map is selected in Model");
            return;
        }

        //Set selected in view
        if(View.ready){

            View.selectMap(oMapNode);
        }

        return oMapNode;
    };
        
    /** Get Selected Map
     *
     */
    this.getSelectMap = function(){
        var oNode = self.getSelectUIComponent();

        //Check is Node
        if(oNode == undefined){
            return;
        }

        //Check is UIComponent
        if(! (oNode.getData() instanceof UIComponent) ){
            console.error("oUIComponent is not a UIComponent instance");
            return;
        }

        var oUIComponent = oNode.getData();

        //Return selected Map
        return oUIComponent.getSelectMap();
    };
        
    /** Deselect Map
     *
     */
    this.deselectMap = function(){
        var oNode = self.getSelectUIComponent();
        var oUIComponent = oNode.getData();

        if(oNode == undefined){
            return;
        }

        var oMapNode = oUIComponent.deselectMap();                                      //Deselect from model

        if(View.ready){
            View.deselectMap(oMapNode);                                                     //Deselect from view
        }
    };
        
        /****** NUOVO LAVORO *******/
    
    /** Map Component (this function don't have its corresponding function in Model or Viewer) */

    /** Array per la selezione multipla (ESPERIMENTO)
     *  TODO esperimento
     * @type {Array}
     */
     var m_aSelectedMapComponents = new Array();
        
    /** Add Component to Map
     *
     */
    this.addComponentToMap = function(oComponentNode,oMapNode,oParam){
        //Check
        if(oComponentNode == undefined){
            console.warn("Specify an oComponentNode to add map");
            return;
        }
        else if(! (oComponentNode.getData() instanceof Component) ){
            console.error("Only a Component instance can be added to a Map");
            return;
        }

        if(oMapNode == undefined){
            console.warn("Specify an oMapNode to add map");
            return;
        }
        else if(! (oMapNode.getData() instanceof Map) ){
            console.error("Component can add only to a Map instance");
            return;
        }

        //Check input parameters
        var oDefaultParam = {
            name: oComponentNode.getData().getParamVal("name"),
            scaleFactor: presenter.preferences.getInterfacePreference("iconsScaleFactor"),
            uiDisableCircle: oComponentNode.getData().getParamVal("uiDisableCircle")
        };

        //Set default parameters
        if(oParam == undefined){
            oParam = oDefaultParam;
        }
        else{
            oParam = $.extend(true, oDefaultParam, oParam);
        }

        var oMap = oMapNode.getData();
        //var oComponent = oComponentNode.getData();

        //Add to model
        var oNode = oMap.addMapComponent(oComponentNode);
        var oMapComponent = oNode.getData();
        var oComponent = oComponentNode.getData();

        //Add to component reference
        oComponent.getParamVal("interfacesLinkedUIComponents").push(oNode);

        //View Mode
        //oUIComponent.addParamVal("viewmode",oRefComponent.getParamVal("viewmodelist")[0]);
        /*if (oComponent.getParamVal("viewmodelist")[0]==undefined)
        {

        }*/
        oMapComponent.viewmode = oComponent.getParamVal("defaultViewMode");
        oMapComponent.viewmode2 = oComponent.getParamVal("defaultViewMode2");
        oMapComponent.viewmode3 = oComponent.getParamVal("defaultViewMode3");
        oMapComponent.viewmode4 = oComponent.getParamVal("defaultViewMode4");
        oMapComponent.viewmode5 = oComponent.getParamVal("defaultViewMode5");


        var aComponentsIcons = oComponent.getParamVal("icons");
        //oMapComponent.icons = new Array();
    //    console.log(oComponent);
    //    console.log(aComponentsIcons);
        if (aComponentsIcons==undefined) return undefined;
        for(var i=0; i<aComponentsIcons.length; ++i){
            var oTmpIcon = new Icon();
            $.extend(true,oTmpIcon,aComponentsIcons[i]);;
            oMapComponent.icons.push(oTmpIcon);
        }

        /*console.error(oMapComponent.icons);
        console.error(oMapComponent.icons);

        console.log(oComponent.getParamVal("icons"));*/

        //Add to component reference
        /*if(oMapComponent.getReference() != undefined){
            //Append this UIComponent node to Component reference
            oRefNode = eveManager.getComponentByUid(oUIComponent.referenceUId);
            oRefComponent = oRefNode.getData();
            oRefComponent.getParamVal("interfacesLinkedUIComponents").push(oNode);
            oUIComponent.setParamVal("name",oRefComponent.getParamVal("name"));
        }*/

        //Subscribe for Model
        $.subscribe("/model/uicomponent/map/mapcomponent/"+ oMapComponent.id +"/update/parameter",PubSub.Model.Update.MapComponent.Parameter);

        //Subscribe for View
        $.subscribe("/view/uicomponent/map/mapcomponent/"+ oMapComponent.id +"/refresh/parameter",PubSub.View.Refresh.MapComponent.Parameter);

        //Set parameters
        for(var k in oParam){
            if(k == "reference"){
                console.error("Set reference to MapComponent");
            }
            else{
                $.publish("/model/uicomponent/map/mapcomponent/"+ oMapComponent.id +"/update/parameter",[oMapComponent,k,oParam[k]]);
            }
        }

        //Add in View
        if(View.ready){
            var $element = View.addMapComponent(oNode,oMapNode);

        }

        return oNode;
    };
        
    /** Remove map component from Map
     *
     */
    this.removeMapComponentFrom = function(oNode,oMapNode){
        //Check
        if(oNode == undefined){
            console.warn("Specify an oNode to remove from map");
            return;
        }
        else if(! (oNode.getData() instanceof MapComponent) ){
            console.error("Only a MapComponent instance can be removed from a Map");
            return;
        }

        if(oMapNode == undefined){
            console.warn("Specify an oMapNode to remove map");
            oMapNode = new Node(oNode.getData().getReferenceMap());
            //return;
        }
        /*else if(!oMapNode.getData() instanceof Map){
            //console.error("MapComponent can add only to a Map instance");
            console.warn("MapComponent can add only to a Map instance");

            if(oMapNode instanceof Map){
                oMapNode = new Node(oNode.getData().getReferenceMap());
            }
            else{
                console.error("MapComponent can add only to a Map instance");
            }
        }*/

        //Remove from model
        var oMap = oMapNode.getData();
        var oMapComponent = oNode.getData();

        var mStatus = oMap.removeMapComponent(oNode);
        if(!mStatus){
            console.error(mStatus);
            return;
        }

        //Remove from Component reference Array
        var oReferenceComponentNode = oMapComponent.getReference();
        var oReferenceComponent = oReferenceComponentNode.getData();
        var aUIComponents = oReferenceComponent.getParamVal("interfacesLinkedUIComponents");
        for(var i=0; i<aUIComponents.length; ++i){
            if(oMapComponent == aUIComponents[i].getData()){
                aUIComponents.splice(i,1);
                break;
            }
        }

        //Unsubscibe
        $.unsubscribe("/model/uicomponent/map/mapcomponent/"+ oMapComponent.id +"/update/parameter");
        $.unsubscribe("/view/uicomponent/map/mapcomponent/"+ oMapComponent.id +"/refresh/parameter");

        if(View.ready){
            View.removeMapComponent(oNode,oMapNode);
        }

        return true;
    };

    /** Select map component
     *
     */
    this.selectMapComponent = function(oNode){

        if(oNode == undefined){
            console.warn("Specify a Node");
            return;
        }
        else if(! (oNode.getData() instanceof MapComponent) ){
            console.error("Only a MapComponent instance can be added to a Map");
            return;
        }

        //
        self.deselectMapComponent();

        //
        oNode = m_aSelectedMapComponents[m_aSelectedMapComponents.push(oNode)-1];

        //Set selected in view
        if(View.ready){

            View.selectMapComponent(oNode);
        }


        //return m_aSelectedMapComponents[m_aSelectedMapComponents.push(oNode)-1];
        return oNode;
    }

    /** Deselect map component
     *
     */
    this.deselectMapComponent = function(){

        while(m_aSelectedMapComponents.length > 0){
            m_aSelectedMapComponents.pop();
        }

        //Set selected in view
        if(View.ready){

            View.deselectMapComponent();
        }

    }

    /** Get selected map component
     *
     */
    this.getSelectMapComponent = function(){

        return m_aSelectedMapComponents[m_aSelectedMapComponents.length-1];
    }

    /** Remove component from Map
     *
     */
    this.removeComponentFromMap = function(oComponentNode,oMapNode){
        console.error("DEPRECATED?!?!?!?!?!?!?!?!?!");
        //TODO DEPRECATED?!?!?!
        return;
        //Get map
        var oData = oComponentNode.getData();
        var oUIComponent = application.getSelectUIComponent().getData();
        //var oMap = oData.referenceMap;
        var $li = $("#interface-maps .maps-container").find(".ui-tabs-active");     //Get active map
        var oMap = $li.data("MapNodeData").getData();


        //Subscribe for Model
        //$.unsubscribe("/model/uicomponent/map/mapcomponent/"+ oMapComponent.id +"/update/parameter");

        //Subscribe for View
        //$.unsubscribe("/view/uicomponent/map/mapcomponent/"+ oMapComponent.id +"/refresh/parameter");

        //Get selected map if
        if(oMap == undefined){
            return;
        }

        for(var i=0; i<oMap.components.length; ++i){

            if(oMap.components[i] == oData){
                oMap.components.splice(i,1);
                break;
            }
        }

        $.publish("/view/uicomponents/"+ oUIComponent.id +"/refresh/maps",[oUIComponent]);
        return;

        if(oMapNode == undefined){
            console.warn("Specify an oMapNode to add map");
            return;
        }
        else if(!oMapNode.getData() instanceof Map){
            console.error("Component can add only to a Map instance");
            return;
        }

        if(oComponentNode == undefined){
            console.warn("Specify an oComponentNode to add map");
            return;
        }
        else if(!oComponentNode.getData() instanceof Component){
            console.error("Only a Component instance can be added to a Map");
            return;
        }

        return;

        var oMap = oMapNode.getData();
        var oComponent = oComponentNode.getData();

        //Check input parameters
        if(oParam == undefined){
            oParam = {
                name: "Untitled"
            };
        }

        $.publish("/view/uicomponents/"+ oUIComponent.id +"/refresh/maps",[oUIComponent]);
        return oMap;
    };

    /** Remove background from map */
    /*this.removeMapBackground = function(oNode){
        var oMap = oNode.getData();

        console.error(oMap);

        //Subscribe for Model
        $.unsubscribe("/model/relations/"+ oRelation.id +"/update/parameter");

        //Subscribe for View
        $.unsubscribe("/view/relations/"+ oRelation.id +"/refresh/parameter");

        if(View.ready){
            View.removeRelation(oNode);
        }
    };*/

    /** Components relations */

    /** Add relations */
    this.addRelation = function(oParam){

        oParam = oParam || {};

        var oDefaultParam = {
            operator: "UNIDIRECTIONAL",
            checkTime: 0
        }

        $.extend(true,oParam,oDefaultParam);

        //Add
        var oNode = model["instance"].addRelation();
        var oRelation = oNode.getData();

        //Subscribe for Model
        $.subscribe("/model/relations/"+ oRelation.id +"/update/parameter",PubSub.Model.Update.Relation.Parameter);

        //Subscribe for View
        $.subscribe("/view/relations/"+ oRelation.id +"/refresh/parameter",PubSub.View.Refresh.Relation.Parameter);

        for(var k in oParam){
            //Set input parameters
            $.publish("/model/relations/"+ oRelation.id +"/update/parameter",[oRelation,k,oParam[k]]);
        }

        if(View.ready){
            View.addRelation(oNode);
        }

        return oNode;
    };

    /** Remove relations */
    this.removeRelation = function(oNode){
        var oRelation = oNode.getData();
        //TODO Farsi ritornare un valore dalla funzione (es. id)
        model["instance"].removeRelation(oNode);

        //Subscribe for Model
        $.unsubscribe("/model/relations/"+ oRelation.id +"/update/parameter");

        //Subscribe for View
        $.unsubscribe("/view/relations/"+ oRelation.id +"/refresh/parameter");

        if(View.ready){
            View.removeRelation(oNode);
        }
    };

    /** Utilities */

    /** Reset Application
     *
     */
    this._reset = function(){
        console.log("%cReset application","color:blue");

        //Clear project
        QT.clearProject();

        eveManager.resetModel();
        View.resetView();

        initModel();

        // Root and default node initialization
        $("#myinterface #level-1").data("eveManagerLinkedNode",eveManager.getUIComponentsRoot());
        $("#myinterface #level-1").data("eveManagerNode",eveManager.getUIComponentsRoot());
        $("#myinterface").data("eveManagerNode",eveManager.getUIComponentsRoot());

        $("#mylists #level-11").data("eveManagerLinkedNode",eveManager.getUIComponentsRoot());
        $("#mylists #level-11").data("eveManagerNode",eveManager.getUIComponentsRoot());
        $("#mylists").data("eveManagerNode",eveManager.getUIComponentsRoot());

        $("#componentscontainer").data("eveManagerNode",eveManager.getComponentsRoot());
        $("#mygateways").data("eveManagerNode",eveManager.getGatewaysRoot());

        //Save icons sets on current project directory (Temporary)
        QT.copyFile("./css/icons-set/images/icons-state_0-128x128.png","../project/assets/icons/icons-state_0-128x128.png");
        QT.copyFile("./css/icons-set/images/icons-state_1-128x128.png","../project/assets/icons/icons-state_1-128x128.png");
    };
        
    /** New project
     *
     */
    this.newProject = function(){
        console.log("%cNew project","color:blue");

        //Reset project
        self._reset();

        //Set curent project to undefined
        presenter.preferences.setParameter("currentProject",undefined);
        presenter.preferences.setInterfacePreferences("iconsScaleFactor",parseFloat( "1" ));
    };
        
    /** Save project
     *
     */
    this.saveProject = function(projectPath){
        console.log("%cSave project","color:blue");
        $document.trigger("app/load/begin",["Saving project"]);
        var oProject = new Object();

        //Application info
        oProject.Application = {
            version: eveManager.version,
            name: eveManager.name,
            iconsPans: new Array(),
            serverIP: model["lastIP"],
            serverPort: model["lastPort"],
            serverSerial: model["lastSerial"],
            skin:model["instance"].skin
        };

        // elenco delle mappe di icone utilizzate

        var aiconsPans=new Array();

        function componentsIconsCheck(oNode){
            try {
                for (var z=0;z<oNode.getData().m_Data.icons.length;z++)
                {
                    var panId=oNode.getData().m_Data.icons[z].iconsPanId;
                    if (aiconsPans.indexOf(panId)==-1)
                    {
                        aiconsPans.push(panId);
                    }
                }

            }catch (e){};


            for(var i1=0;i1<oNode.children.length;++i1){
                componentsIconsCheck(oNode.children[i1]);
            }
        };

        var oComponentsRoot = eveManager.getComponentsRoot();
        for(var i2=0;i2<oComponentsRoot.children.length;++i2){
            componentsIconsCheck(oComponentsRoot.children[i2]);
        }
        console.log("Icons pans found");
        console.log(aiconsPans);

        var aPans = self.ResoucesManager.getIconsPans();
        for(var i=0; i<aPans.length; ++i){
            // if the one icons of the iconspan is used i need to copy the files to the project folder
            // and save the iconspan insode the project itself.
            // if not used i don't put it in the project

            // cicla tra i componenti
            //Declare recursive function for decode components

            var srcFileName= aPans[i].params.src.split('/').pop();
            oProject.Application.iconsPans.push({
                id: aPans[i].id,
                Pan:aPans[i],
                //name: aPans[i].params.name,
                path: "/project/assets/icons/"+ srcFileName,
                pathSelected: "/project/assets/icons/"+ "selected_"+srcFileName
            });
            console.log("Img path: /project/assets/icons/"+ aPans[i].params.name);

            // copia il file nella cartella del progetto

            // se il set di icone fa parte delle icone di EVE Manager il path per la copia del file cambia


            if (aPans[i].params.setType=="default")
            {
                QT.copyFileAbsolute(QT.applicationPath()+"/data/html/assets/icons/ilevia/"+srcFileName,QT.applicationPath()+"/data/project/assets/icons/"+srcFileName)
                QT.copyFileAbsolute(QT.applicationPath()+"/data/html/assets/icons/ilevia/"+ "selected_"+srcFileName,QT.applicationPath()+"/data/project/assets/icons/"+ "selected_"+srcFileName)
            }
            else
            {
                QT.copyFileAbsolute(QT.getStoragePath()+"/assets/icons/custom/"+srcFileName,QT.applicationPath()+"/data/project/assets/icons/"+srcFileName)
                QT.copyFileAbsolute(QT.getStoragePath()+"/assets/icons/custom/"+ "selected_"+srcFileName,QT.applicationPath()+"/data/project/assets/icons/"+ "selected_"+srcFileName)

            }
           //

        }
        //console.log(self.ResoucesManager.getIconsPans());
        //console.log(oProject.Application.iconsPans);

        //Serialize gateways
        oProject.Gateways = new Array();

        var oGatewayRoot = eveManager.getGatewaysRoot();
        for(var i=0;i<oGatewayRoot.children.length;++i){
            var oNode = oGatewayRoot.children[i];
            //Push serialization object
            oProject.Gateways.push(oNode.encodeJSON());
        }

        //Serialize components
        oProject.Components = new Array();

        //Declare recursive function for decode components
        function componentsSerialize(oNode){

            //Push serialization object
            oProject.Components.push(oNode.encodeJSON());

            for(var i=0;i<oNode.children.length;++i){
                componentsSerialize(oNode.children[i]);
            }
        };

        var oComponentsRoot = eveManager.getComponentsRoot();
        for(var i=0;i<oComponentsRoot.children.length;++i){
            componentsSerialize(oComponentsRoot.children[i]);
        }


        // create json for classic UI




        //Save current User
        self.saveUser();

        //Serialize users
        oProject.Users = new Array();

        //Existing images array
        var aSavedImgs = QT.fileList("./data/project/assets/img/backgrounds");
        //var aSavedImgs = QT.fileList("./data/project/assets/img/ipad");
        var aUsedImgs = new Array();

        //Push serialization object
        var oUsersRoot = eveManager.getUsersRoot();
        for(var i=0;i<oUsersRoot.children.length;++i){
            var oNode = oUsersRoot.children[i];
            var oUser = oNode.getData();
            var oJson = oNode.encodeJSON();

            //Skip if image already exist
            if(aUsedImgs.indexOf(oUser.backgroundImage) == -1){
                aUsedImgs.push(oUser.backgroundImage);
            }

            //Push serialization object
            oProject.Users.push(oJson);

            //console.warn(oUser.name);
            for(var j=0; j<oJson.UI.length; ++j){
                if(oJson.UI[j].parentUid != undefined){
                    continue;
                }

                //console.error(oJson.UI[j]);

                var aMaps = oJson.UI[j].maps;
                for(var k=0; k<aMaps.length; ++k){
                    //Skip if background name is empty
                    if(aMaps[k].background.length < 1){
                        continue;
                    }

                    //Skip if image already exist
                    if(aUsedImgs.indexOf(aMaps[k].background) != -1){
                        continue;
                    }

                    //console.log("PUSH: "+ aMaps[k].background);
                    aUsedImgs.push(aMaps[k].background);
                }
            }
            // } IN DEBUG
        }

        //Remove unused images
        //--console.log(aUsedImgs);
        //--console.log(aSavedImgs);

        //Remove unused images
        while(aSavedImgs.length > 0){
            var bFound = false;
            //var sFilePath = "../project/assets/img/ipad/";
            var sFilePath = "../project/assets/img/backgrounds/";

            for(var i=0;i<aUsedImgs.length;++i){
                if(aUsedImgs[i].indexOf(aSavedImgs[0].name) != -1){
                    bFound = true;
                    break;
                }
            }

            //Not found
            if(!bFound){
                //Remove file
                //console.error("Remove: "+ sFilePath + aSavedImgs[0].name);
                //console.error("Remove: "+ aSavedImgs[0].name);
                QT.deleteFile(sFilePath + aSavedImgs[0].name);
            }

            //Remove element from array
            aSavedImgs.splice(0, 1);
        }//end while

        //Save relations
        oProject.Relations = new Array();

        var oRelationsRoot = model.instance.getRelationsRoot();
        for(var i=0;i<oRelationsRoot.children.length;++i){
            var oNode = oRelationsRoot.children[i];

            //Push serialization object
            oProject.Relations.push(oNode.encodeJSON());
        }

        //console.log(oProject);

        //Project JSON and project path (if exist)
        var oResponse = QT.saveProject(JSON.stringify(oProject),projectPath,presenter.preferences.getParameter("projectsDir"));

        //--console.log(oResponse);
        if(oResponse.status != "ok"){
            if(oResponse.status != "cancel"){
                $document.trigger("app/error",["Can't save current project"]);
            }
            console.error(oResponse.status);
            $document.trigger("app/load/end");
            return;
        }

        //Add loaded project to the beginning of array
        var oCurrentProject = {
            projectName: oResponse.name,
            projectPath: oResponse.path
        };

        if(oCurrentProject.projectName != "temp_upload.eve"){
            //Set current project
            presenter.preferences.setParameter("currentProject",oCurrentProject);

            //Add current project to recent projects list
            presenter.preferences.addRecentProject(oCurrentProject);
        }

        $document.trigger("app/load/end");

        //Graphic effect
        /*
        setTimeout(function(){
            var $img = $("<img src='./assets/img/save-icon.png' height='160' width='160' style='position: absolute; bottom: 50px; right: 50px; font-size: 32px; background-color: #222; color: yellow; padding: 30px; border-radius: 30px; opacity: 1; display: block;' />");
            $("body").append($img);
            $img.animate({ opacity:.9 },200).animate({ opacity: 0 },{
                duration: 200,
                complete: function(){
                    $img.remove();
                }
            });
        });
        */
        return oProject;
    };

    /** Pre-load project
     * @param {string|undefined} projectPath
     */
    this.preLoadProject = function(projectPath){

        //Load project
        //var oResponse = QT.loadProject(projectPath);
        //alert("la");
        //return;
        console.log("Preload");
        var oResponse = QT.loadProjectConfig(projectPath,presenter.preferences.getParameter("projectsDir"));

        $document.trigger("app/load/begin",["Loading project"]);

        //Check response
        if(oResponse.status.toLowerCase() != "ok"){
            //End loader
            $document.trigger("app/load/end");

            //End loader
            if(oResponse.status.toLowerCase() == "error"){
                $document.trigger("app/error",["Can't load selected project"]);
            }
            return;
        }

        //Reset current project

        //End loader
        $document.trigger("app/load/end");
        presenter.getWindow("databaseManager").show(undefined, {
            "command": "load",
            "project": { projectName: oResponse.name, projectPath: oResponse.path },
            "json": JSON.parse( oResponse.project )
        });
    };

    /** Load project
     * @param projectPath
     */
    this._loadProject = function(oProject){
        console.log("%cLoad project","color:blue");

        if(typeof oProject != "object"){
            $document.trigger("app/error",["Can't load selected project"]);
            return;
        }

        model["lastIP"] = oProject.Application.serverIP;
        model["lastPort"] = oProject.Application.serverPort;
        model["lastSerial"] = oProject.Application.serverSerial;

        var iPans=oProject.Application.iconsPans;
        for (var i=0;i<iPans.length;i++)
        {
            var projectPan=iPans[i];
            console.log(projectPan.id);
            if (application.ResoucesManager.getIconsPanById(projectPan.id))
            {
                console.log ("Icons already present");

            }
            else
            {
                //verifica se sono state utilizzate icone non presenti in quelle installate attualmente
                // nel caso ce ne siano di nuove aggiungile al pan di icone
                console.log("Icons does not exists, adding "+projectPan);
                console.log(JSON.stringify(projectPan));

                application.ResoucesManager.addIconsPanFromDownoadedProject(projectPan);



            }
        }




        //Empty Model and View (Start new project)
        //self.newProject();
        //self._reset();

        //setTimeout(function(){ $document.trigger("app/load/begin",["Loading project"]); },1);

        //Build Gateways
        for(var i=0; i<oProject.Gateways.length; ++i){
            var gateway = oProject.Gateways[i];

            if(gateway.special != undefined){
                continue;
            }

            self.addGateway(gateway);
        }

        //Select first gateway
        self.selectGateway(eveManager.getGatewaysRoot().children[0]);

        //
        //setTimeout(function(){

        //Build Components
        for(var i=0; i<oProject.Components.length; ++i){
            var component = oProject.Components[i];
            var oParent = eveManager.getComponentByUid(component.parentUId);

            if(component.special != undefined){
                continue;
            }


            var oNode = self.addComponent(component,oParent,true);

            if(oNode == undefined){
                console.error("can't add new component");
                console.error(component);
            }
        }

            //Empty default users
            var oUsersRoot = eveManager.getUsersRoot();
            while(oUsersRoot.children.length > 0){
                self.removeUser(oUsersRoot.children[0]);
            }

            //Build Users
            for(var i=0; i<oProject.Users.length; ++i){
                //console.error(oProject.Users[i]);
                var oParentUser = self.getUserByName(oProject.Users[i].createdBy);

                var oNode = self.addUser(oProject.Users[i],oParentUser);

                //console.error(oNode.encodeJSON());
            }

            //Select connected user
            //self.selectUser( self.getUserByName( presenter.preferences.getParameter("connectedUser") ) );
            //console.log(model["connectedUser"].getData());
            self.selectUser( self.getUserByName( model["connectedUser"].getData().name ) );

            //Build relations
            for(var i=0; i<oProject.Relations.length; ++i){
                var oRel = oProject.Relations[i];

                oRel["source"] = eveManager.getComponentByUid(oRel["source"]);
                oRel["destination"] = eveManager.getComponentByUid(oRel["destination"]);

                self.addRelation(oRel);
            }



            //console.log(oProject);
            //$.publish("/view/components/refresh/dependencies");

        //},1);

        //console.log(oProject);
    //    $.publish("/view/components/refresh/dependencies");
        $document.trigger("app/load/end");

        //REFRESH AND SELECT
        // per fare il refresh dei componenti
        // fixme: scoprire come generare l'activate della tab
        $("#tabs").tabs( "refresh" ).find('[href="#gateways"]').trigger("click");
        $("#tabs").tabs( "refresh" ).find('[href="#createcomponents"]').trigger("click");



        return oProject;
    };

    this.loadProject = self._loadProject;

    /** Get system info */
    this.getSystemInfo = function(){
        //Print in console and open dialog box
        var $info = $("#system-info");
        var $content = $info.find("p");
        $content.empty();
        $content.append("<ul></ul>");
        for(var k in informations){
            console.log("%c"+ k +": %c"+ informations[k],"font-size:14px; ","font-size:inherit; color:blue;");
            switch(k){
                case "required":
                case "author":
                    continue;
                    break;
            }

            $content.find("ul").append("<li style='margin:0.5em;'><strong>"+ k.capitalizeFirst() +"</strong>: "+ informations[k] +"</li>");
        }

        $content.find("ul").append("<li style='margin:0.5em;'><strong>QT Release</strong>: "+ QT.getQTRelease() +"</li>");

        $info.dialog("open");
    };

    /** Check if project is empty */
    this.isEmpty = function(){

        var oRoot = eveManager.getGatewaysRoot();
        if(oRoot.children.length > 4){
            return false;
        }

        var oRoot = eveManager.getComponentsRoot();
        if(oRoot.children.length > 2){
            return false;
        }

        var oRoot = eveManager.getUIComponentsRoot();
        if(oRoot.children.length > 0){
            return false;
        }

        return true;
    };
        
	// Call presenter initial operations
	//init();
};