﻿/** Resources Manager Controller: Business logic singleton class
 *
 * @class ResourcesController
 *
 *
 * @author <a href="mailto:info@ilevia.com">info@ilevia.com</a>
 *
 * 
 * @requires jQuery framework
 */
//"use strict";

function ResourcesController(oResourseManager) {
    //Build singleton instance
    if (arguments.callee._singletonInstance) {
        console.warn("Singleton instance used");
        return arguments.callee._singletonInstance;
    }

    arguments.callee._singletonInstance = this;

    /** Instance variables (private) */
    var self = this;

    /** Resource Manager reference
     *
     */
    var m_oReference = oResourseManager;

    /** Initialize private method
     *
     */
    var init = function () {

    };

    /**
     *
     */
    this.addIconsPan = function (oParams) {
        // se esiste non lo aggiungo
        var pan=this.getIconsPanById(oParams.id);

        if (!pan)
        {
            var oPan = m_oReference.addSprite($.extend(false, oParams, {
                "type": "pan/icons"
            }));

            /*
             QT.copyFile(oPan["params"]["src"], "../project/assets/icons/" + oPan["params"]["name"]);
             if (oPan["params"]["srcSelected"] != "")
             QT.copyFile(oPan["params"]["srcSelected"], "../project/assets/icons/" + "selected_" + oPan["params"]["name"]);
             */

            if (!View.ready) {
                return;
            }

            View.addIconsPan(oPan);

        }
        return;
    };

    this.removeIconsPan = function (id) {
        m_oReference.removeSprite(id);
    };
    /**
     *
     */
    this.getDefaultIconsPan = function (argument) {
        return m_oReference.getSpritesByType("pan/icons")[0];
    };

    /**
     *
     */
    this.getIconsPanById = function (id) {
        var ipan=m_oReference.getSpriteById(id);
        if (!ipan)
            return self.getIconsPanByParamId(id);
        return ipan;
    };

    this.getIconsPanByParamId = function (id) {
        return m_oReference.getSpriteByParamId(id);
    };

    /**
     *
     */
    this.getIconsPans = function () {
        return m_oReference.getSpritesByType("pan/icons");
    };

    this.removeIconsPanByParameterId=function (id) {
        var customIcons = new Array();
        try {
            customIcons = JSON.parse(QT.readAbsoluteFile(QT.getStoragePath() + "/assets/icons/custom/iconsPan.json"));
            for (var x=0;x<customIcons.length;x++)
            {
                if (customIcons[x].id==id)
                {
                    console.log("Removing iconspan: "+id);
                    console.log(customIcons[x]);
                    customIcons.splice(x,1);
                    QT.writeAbsoluteFile(QT.getStoragePath() + "/assets/icons/custom/iconsPan.json", JSON.stringify(customIcons));
                    View.removeIconsPan(id);
                    this.removeIconsFromModel(id);
                    break;
                }

            }

        }
        catch (e) {
            return false;
        }
        console.log(customIcons[x]);
    };

    this.addIconsPanFromDownoadedProject = function (projectPan) {

        if (projectPan.Pan==undefined) return;

        var newIconsPan = {

            "name": projectPan.Pan.params.name,
            "id": projectPan.id,
            "visible": true,
            "description": projectPan.Pan.params.name,
            "label": projectPan.Pan.params.name,

            "src": ".."+projectPan.path,
            "srcSelected":".."+projectPan.pathSelected,
            "imgHeight": projectPan.Pan.params.imgHeight,
            "imgWidth": projectPan.Pan.params.imgWidth,
            "iconsHeight": projectPan.Pan.params.iconsHeight,
            "iconsWidth": projectPan.Pan.params.iconsWidth

        }
        this.addIconsPan(newIconsPan);
    };

    this.refreshIconsFromStorage=function () {
        // salva il nuovo skin

        var customIcons=new Array();
        try{
            customIcons=JSON.parse( QT.readAbsoluteFile(QT.getStoragePath()+"/assets/icons/custom/iconsPan.json"));

        }
        catch (e){customIcons=new Array();}

        for (var i=0;i<customIcons.length;i++)
        {
            application.ResoucesManager.addIconsPan(customIcons[i]);
        }
    };

    this.addIconsFromModel=function() {
        var pans=model["instance"].skin.IconsPan;
        for(var i=0; i<pans.length; ++i){
            this.addIconsPan(pans[i]);
        }
    };

    this.removeIconsFromModel=function(iconPanId) {
        application.ResoucesManager.removeIconsPan(iconPanId);
        $.publish("/view/application/refresh/icons");
    };


    this.addIconsPanFromFile = function (selectedFilename, name, iconWidth, iconHeight) {
        // copia il file delle icone dentro gli assets nello storage
        var localFilePath = QT.copyAbsoluteFileInEVEManager(selectedFilename, QT.getStoragePath() + "/assets/icons/custom/");

        // aggiunge nella cartella del progetto il json con i dati necessari
        var fileName = localFilePath.split('/').pop();
        var storageFileName = QT.getStoragePath() + "/assets/icons/custom/" + fileName;
        var storageFileNameSelected = QT.getStoragePath() + "/assets/icons/custom/" +"selected_"+fileName;

        console.log("Absolute file name:" + storageFileName);
        var imageInfo = QT.getImageInfo(storageFileName);
        var newIconsPan = {

            "name": name,
            "id": fileName,
            "visible": true,
            "description": name,
            "label": name,

            "src": localFilePath,
            "srcSelected":storageFileNameSelected,
            "imgHeight": imageInfo.height,
            "imgWidth": imageInfo.width,
            "iconsHeight": iconHeight,
            "iconsWidth": iconWidth

        }
        console.log(newIconsPan);
        application.ResoucesManager.addIconsPan(newIconsPan);

        //Add created icons pan
        var aPans = model["instance"].ResourcesManager.getSpritesByType("pan/icons");
        //--console.log(aPans);
        // se non è presente nella view lo aggiungiamo
        var found=false;
        for (var i = 0; i < aPans.length; ++i) {
            if (aPans[i].params.name == fileName)
            {
                found=true;
                break;
            }

        }
        if (found==false)
        {
            //       View.addIconsPan(newIconsPan);
        }

        // aggiorna il file per partire con le icone aggiunte
        model["instance"].skin.IconsPan.push(newIconsPan);
        // salva il nuovo skin
        var customIcons = new Array();
        try {
            customIcons = JSON.parse(QT.readAbsoluteFile(QT.getStoragePath() + "/assets/icons/custom/iconsPan.json"));
        }
        catch (e) {
            customIcons = new Array();
        }

        console.log("customIcons folder");
        console.log(customIcons);
        console.log(newIconsPan);
        customIcons.push(newIconsPan);

        console.log(customIcons);
        QT.writeAbsoluteFile(QT.getStoragePath() + "/assets/icons/custom/iconsPan.json", JSON.stringify(customIcons));



    };


    /** Initialize */
	init();
}