/** scenarioEditor class (extends Window class)
 *
 * @class scenarioEditor
 *
 *
 * @author <a href="mailto:info@ilevia.com">info@ilevia.com</a>
 *
 */
"use strict";

function scenarioEditor(oWindow){
	
	var self = this;
	
	/** Initialization
	 *
	 */
	var init = function(){
        console.log("%cWindow - scenarioEditor class initialization (extends Window class)","color:green");
        
        //Create jQuery UI dialog
        m_$element.dialog({
            height: 700,
            minHeight: 700,
            width: 1000,
            minWidth: 1000,
            autoOpen: false,
            title: _D("Scenario composition"),
            closeText: _D("save and close"),
            modal: true,
            create: function(e,ui){
                var $this = $(this);
                var $components = $this.find(".components");
                var $composition = $this.find(".composition");
                
                //$this.find("> div").w2layout({
                $this.w2layout({
                    name: "ghost-scenario-editor",
                    panels: [
                        { type: 'left', size: 500, minSize: 500, resizable: true, style: "background-color: #333; border: 1px solid #dfdfdf; padding:0px; z-index:auto;", content: $components[0] },
                        { type: "main", minSize: 200, style: "background-color: #333; border: 1px solid #dfdfdf; padding:0px; z-index:auto;", content: $composition[0], overflow: "hidden" }
                    ]
                });
                
                //Dinamically hide overflow
                $("#layout_ghost-scenario-editor_panel_main").css("overflow","hidden");
            },
            resize: function(e,ui){
                var $this = $(this);
                var intOffset = 119;
                
                $this.find(".actions-box").height(parseInt($this.height(),10) - intOffset);
            },
            open: function(e,ui){
                var $this = $(this);
                var intOffset = 119;
                
                $this.trigger("resize");
                $this.find(".actions-box").height(parseInt($this.height(),10) - intOffset);
            },
            close: m_oWindow.hide
        });
        
        //Droppable attach component
        m_$element.find(".attachcomponent").droppable({
            //accept: m_$element.find(".components [eve-manager-component='switch'], .components [eve-manager-component='lock'], .components [eve-manager-component='pulse']"),
            //accept: "#scenarioEditor .components [eve-manager-component='switch'], #scenarioEditor .components [eve-manager-component='power'], #scenarioEditor .components [eve-manager-component='lock'], #scenarioEditor .components [eve-manager-component='pulse']",
            //accept: "[eve-manager-component='switch'], [eve-manager-component='lock'], [eve-manager-component='pulse']",
            //accept: "[eve-manager-component='switch'], [eve-manager-component='lock'], [eve-manager-component='pulse'], [eve-manager-component='timer']",
            activeClass: "accept",
            drop: self._setAttachedComponent
        });
        
        //Empty attached
        m_$element.on("click",".attachcomponent .ui-icon-trash",function(e){            
            $(this).parents(".attachcomponent").empty();
        });
        
        //Ghost - scenario composition
        m_$element.find(".actions").sortable($.extend(false,basicSortable),{
            //appendTo: "#scenarioComposition .actions-box",
            //containment: "#scenarioComposition .actions-box",
            //connectWith: "#scenarioComposition .components",
            
            //Check utility
            appendTo: m_$element.find(".actions-box"),
            //containment: m_$element.find(".actions-box"),
            connectWith: m_$element.find(".components"),
            forcePlaceholderSize: true,
            scroll: true,
            //receive: self._addActionComponent
            receive: function(e,ui){
                var $this = $(this);
                var $helper = $this.find("[eve-manager-component]");
                var oNode = ui.item.data("eveManagerNode");
                var $element = self._addAction(oNode);
                
                $helper.before($element);                                                                   //Append html element
        
                //Remove helper
                $helper.remove();
            }
        });
        
        //Remove action
        m_$element.on("click",".actions .ui-icon-trash",function(e){
            $(this).parents("li").remove();
        });


	    //Get filters
	    //fixme:Cambiare con QT.readFile
        /*
        $.ajax({
            dataType: "json",
            url: "./declarations/scenario/ghost/attached-components.json",
            async: false,
            success: function(aData){
                
                var str = "";
                for(var i=0; i<aData.length; ++i){
                    str += "[eve-manager-component='"+ aData[i] +"'],";
                }
                str = str.substring(0, str.length - 1);
                
                m_$element.find(".attachcomponent").droppable("option", "accept", str);
            },
            error: function(jqXHR, textStatus, errorThrown){
                console.error(errorThrown);
                m_oWindow.hide();                                   //Call external window hide method
            }
        });
        */
        var aData = $.parseJSON(QT.readFile("./declarations/scenario/ghost/attached-components.json"));
        var str = "";
        for (var i = 0; i < aData.length; ++i) {
            str += "[eve-manager-component='" + aData[i] + "'],";
        }
        str = str.substring(0, str.length - 1);
        m_$element.find(".attachcomponent").droppable("option", "accept", str);

	    //Get black list components
        self["black-list"] = $.parseJSON(QT.readFile("./declarations/scenario/ghost/black-list.json"));
    };
	
	/** Extended window object 
     * @type {Window}
     */
	var m_oWindow = oWindow;
	
	/** jQuery DOM element representing Window GUI
     * @type {jQuery}
     */
    var m_$element;
    
	/** Component action
     * @type Array
     */
    this["black-list"] = new Array();
	
	/** Show icon editor window
	 */
	this.show = function(oParam){
	    //Check Component instance
	    if(!m_oWindow.getActiveNode().getData() instanceof Component){
	        console.error("Active node is not Component instance");
	        return;
	    }
	    
	    //Get component
	    var oNode = m_oWindow.getActiveNode();
	    var oComponent = oNode.getData();
	    var oComposition = oComponent.getParamVal("ghost");
	    
	    //Check Component is scenario type
        if(oComponent.type != "scenario"){
            console.error("Active node is not a scenario component");
            return;
        }
	    
	    //Clone components inside scenario
	    var $originalContainer = $("#componentscontainer");                                    //Get original components container
        var $clonedComponents = $originalContainer.children().clone(true,true);                //Get cloned html components
	    var $componentsContainer = m_$element.find(".components");
	    
	    //Fill avaiable components
	    $componentsContainer.empty();
	    $componentsContainer.append($clonedComponents);
	    
	    var $components = $componentsContainer.find("[eve-manager-component]");
	    $components.removeClass("select").removeClass("ui-state-active").removeAttr("id");
        //$components.find("canvas").attr("width",32).attr("height",32);

        //Remove sortable
        //$componentsContainer.find("[eve-manager-component='group'] > ul").css("border","1px red solid");//.sortable("destroy");
        //$componentsContainer.find("[eve-manager-component='group'] > ul").sortable("destroy");

        //$componentsContainer.find(".ui-sortable")//.removeClass("ui-sortable")
        //alert($componentsContainer.find(".ui-sortable").length);
	    /*
	    //Draw icons
	    for(var i=0; i<$components.length; ++i){
	        var $current = $($components[i]);
	        var oNode = $current.data("eveManagerNode");
	        var oComponent = oNode.getData();
	        var aIcons = oComponent.getParamVal("icons");
            
	        if(aIcons == undefined){
	            continue;
	        }
	        
	        var oIcon = aIcons[0];
            //var img = document.getElementById("../project/assets/icons/icons-state_1-128x128.png");
            var img = document.getElementById(oIcon.srcId);

	        var imgHeight = parseInt(img.height,10);                        // Image height
            var imgWidth = parseInt(img.width,10);                          // Image width
	        var stretchScale = 0.25;
	        var drawHeight = imgHeight * stretchScale;
            var drawWidth = imgWidth * stretchScale;
	        var cont = $current.find("canvas")[0].getContext("2d");
	        
	        //Height - Width streched
            cont.drawImage(img,(oIcon.position.x * stretchScale),(oIcon.position.y * stretchScale),drawWidth,drawHeight);
            
            var imageData = cont.getImageData(0,0,oIcon.size.w*stretchScale,oIcon.size.w*stretchScale);
                                
            //Color image
            if(oIcon.enableColors){
                //Color image
                for(var j=0; j<imageData.data.length; j+=4){
                    imageData.data[j+0] = oIcon.color.R;
                    imageData.data[j+1] = oIcon.color.G;
                    imageData.data[j+2] = oIcon.color.B;
                    imageData.data[j+3] = imageData.data[j+3];
                }
            }
            
            cont.putImageData(imageData,0,0);
	    }*/
	    
	    //Set draggable component
        $componentsContainer.find("[eve-manager-component]:not([eve-manager-component='group'])").draggable({
            appendTo: "body",
            containment: "body",
            revert: "invalid",
            revertDuration: false,
            connectToSortable: m_$element.find(".attachcomponent, .actions"),
            helper: function(e){ return $(e.currentTarget).clone(true); },
            start: function(e,ui){
                m_$element.find(".actions-box").addClass("accept");
            },
            stop: function(e,ui){
                m_$element.find(".actions-box").removeClass("accept");
            },
            zIndex: 1000
        });

	    //Hide black-list components
	    for(var i=0; i<self["black-list"].length; ++i){
            
            if(self["black-list"][i]["parameter"] != undefined){
                var $componentByType = $componentsContainer.find("[eve-manager-component='"+ self["black-list"][i]["type"] +"']");
                
                for(var j=0; j<$componentByType.length; ++j){
                    var oNode = $( $componentByType[j] ).data("eveManagerNode");
                    var oComponent = oNode.getData();
                    
                    switch(self["black-list"][i]["parameter"]){
                        case "gateway":
                            if(oComponent.getParamVal(self["black-list"][i]["parameter"]).getData().getProtocol() == self["black-list"][i]["value"]){
                                $( $componentByType[j] ).hide();
                            }
                            break;
                        default:
                            if(oComponent.getParamVal(self["black-list"][i]["parameter"]) == self["black-list"][i]["value"]){
                                $( $componentByType[j] ).hide();
                            }
                    }
                }
            }
            else{
                $componentsContainer.find("[eve-manager-component='"+ self["black-list"][i]["type"] +"']").hide();
            }
            
        }
	    
	    //var oComposition = oComponent.getParamVal("ghost");
	    
	    //Fill attached component
        if(oComposition != undefined && oComposition.attached != undefined && oComposition.attached.UId != undefined){
            var $attachedTo = m_$element.find(".attachcomponent");
            var $element = getScenarioAttachHtmlElement();
            
            $attachedTo.append($element);                                                      //Append element to list
            
            //Set parameters
            var oAttachedNode = application.getComponentByUid( oComposition.attached.UId );
            var oAttachedComponent =  oAttachedNode.getData();
            var oParentAttachedNode = oAttachedNode.getParent();
            //var oAttachedParent = oParentAttachedNode.getData();
            
            $element.find(".category-txt").text(((oParentAttachedNode.getParamVal != undefined)?oParentAttachedNode.getParamVal("name"):_D("none")));
            $element.find(".name-txt").text(oAttachedComponent.getParamVal("name"));
            $element.find(".loop").prop("checked",oComposition.attached.loop);
            
            //Link node data
            $element.data("eveManagerNode",oAttachedNode);
        }
        
        //Fill componens actions
        if(oComposition != undefined && oComposition.actions != undefined){
            var $actions = m_$element.find(".actions");
            
            for(var i=0; i<oComposition.actions.length; ++i){
                var oTempNode = application.getComponentByUid( oComposition.actions[i].UId );
                var oTempComponent = oTempNode.getData();
                var oTempParent = oTempNode.getParent();
                var oTempParentComponent = oTempParent.getData();
                var oDescriptor = oTempComponent.getDescriptor();
                var $element = self._addAction( application.getComponentByUid( oComposition.actions[i].UId ));
               
                //Set element parameters
                $element.find(".category").text((oTempParentComponent.getParamVal)?oTempParentComponent.getParamVal("name"):_D("none"));
                $element.find(".name").text(oTempComponent.getParamVal("name"));
                $element.find(".value").val(oComposition.actions[i].value);

                if(oTempComponent.getType() == "rgb-light") {
                    console.log($element.find(".value").val(oComposition.actions[i].value));
                    $element.find(".value").css("background-color","#"+ oComposition.actions[i].value);
                }
                
                var $componentActions = $element.find(".actions");
                
                $componentActions.find("[value='"+ oComposition.actions[i].action +"']").prop("selected",true);
                
                $element.data("eveManagerNode",oTempNode);
                $actions.append($element);
            }
        }
	    
	    m_$element.dialog("open");
	};
	
	/** Hide icon editor window and closing operation
     *
	 */
	this.hide = function(e,ui){
        var $components = m_$element.find(".components");
        var $attachcomponent = m_$element.find(".attachcomponent");
        var $actionscomponents = m_$element.find(".actions");
        var $attached = $attachcomponent.children();
        var $actions = $actionscomponents.children("li");
        var serialized = new Object();
        
        //Attached component
        if($attached.length > 1){
            console.error("More than one component attached to scenario");
            return;
        }
        
        //Serialize attached element
        var oAttachedNode = $attached.data("eveManagerNode");
        serialized.attached = {
            //UId: $attached.data("eveManagerNode").getData().UId,                           //Set Node component
            UId: (oAttachedNode != undefined)?oAttachedNode.getData().UId:undefined,
            loop: $attached.find(".loop").is(':checked'),                                   //Set boolean loop
            reverse: $attached.find(".reverse").is(':checked')                              //Set boolean reverse
        };
            
        //Actions component
        serialized.actions = new Array();
        
        for(var i = 0; i < $actions.length; ++i){
            var $current = $($actions[i]);
            var parameters = new Object();
            var oComponent = $current.data("eveManagerNode").getData();
            var oDescriptor = oComponent.getDescriptor();
            //console.log("-----------------");
            //console.log(oDescriptor);
            //Parameters
            parameters = {
                UId: oComponent.UId,
                action: $current.find("select :selected").val(),
                //Change value representation if required
                value: $current.find("input[type='text']").val()
            };
            console.log(parameters);
            //TODO in test
            if(!oDescriptor.isValueEditable()){
                //console.error(oComponent.getType());
                //console.error("Value: "+ parameters.value);
                //console.error("Action: "+ parameters.action);
                //console.error("Action: "+ parameters.action);
                parameters.value = parameters.action;
            }
            console.log($current.find("input[type='text']").val());
            console.log(oDescriptor.getDescriptionByValue($current.find("input[type='text']").val()));
            
            serialized.actions.push(parameters);
        }
        
        //Add serialized composition
        var oComponent = m_oWindow.getActiveNode().getData();
        oComponent.setParamVal("ghost",serialized);                                         //Set ghost parameter

        //console.error(serialized);
        /*
        //Check correct
        $("#componentproperties .container").html(QT.readFile("./components/"+ oComponent.getType() +"/"+ oComponent.getType() +".html"));
        onUpdateComponent("Window-scenarionEditor",COMPONENT);
        */

        var eventData = new blrComponentEventData(
            "Window-scenarionEditor.js",
            $("#componentproperties"),
            oComponent
        );
        eventData.$properties().find("[eve-manager-components='mainframe']").html(QT.readFile("./components/mainframe/components-mainframe.html"));
        eventData.$properties().find("[eve-manager-components='bottom']").html(QT.readFile("./components/mainframe/bottom-prop.html"));
        COMPONENTS_MAINFRAME.onGuiChangeGatewayCallbackUserData = eventData;
        COMPONENTS_MAINFRAME.onGuiChangeGatewayCallback = onGuiChangeGateway;
        onInitComponent(eventData);
        onUpdateComponent(eventData);

        eventData = null;

        //Empty html
        $attachcomponent.empty();                                                           //Empty attached component container
        $actionscomponents.empty();                                                         //Empty actions container
        
        //console.log(serialized);
	};
	
	/** Assign window HTML element 
     * @param $element jQuery element represent this window
     * @returns {Node}  
     */
    this.assignWindow = function($element){
        m_$element = $element;
        
        //Initialization after graphic ready
        init();
    };
    
    /** Interactions methods */
    
    /** Set attached component
     * 
     */
    this._setAttachedComponent = function(e,ui){
        var $this = $(this);
        //var $element = $this.children("[eve-manager-component]");
        var $element = ui.helper;
        var $attached = $this.children(".attached");
        var oNode = $element.data("eveManagerNode");
        var oParentNode = oNode.getParent();
        var oComponent = oNode.getData();
        var oParentData = oParentNode.getData();
        
        //Check attached element existance (create element html)
        if($attached.length < 1){
            $attached = getScenarioAttachHtmlElement();                                         //Create new
            $this.append($attached);                                                            //Append element to list
        }
        
        var strCategory = (oParentData.getParamVal)?oParentData.getParamVal("name"):_D("none");
        $attached.find(".category-txt").text(strCategory);                                      //Set attached element category
        $attached.find(".name-txt").text(oComponent.getParamVal("name"));                       //Set attached element name
        
        $attached.data("eveManagerNode",oNode);
    };
    
    /** Add action component
     * 
     */
    this._addAction = function(oNode){
        var oComponent = oNode.getData();
        var oParentNode = oNode.getParent();
        var oParentComponent = oParentNode.getData();
        var oDescriptor = oComponent.getDescriptor();
        var $element = getScenarioActionHtmlElement();


        
        //Set element parameters
        $element.find(".category").text((oParentComponent.getParamVal)?oParentComponent.getParamVal("name"):_D("none"));
        $element.find(".name").text(oComponent.getParamVal("name"));
        $element.find(".value").val(oDescriptor.getDefaultValue());


        if(oComponent.getType() == "rgb-light"){

            $element.find(".value").css('border-right-color',"#"+oDescriptor.getDefaultValue()).click(function(){
                var $colorElement = $(this);
                blr.colorPicker.onChange = function(hsb,hex,rgb,el,bySetColor){
                    // $colorElement.css("background-color","#"+ hex);
                    $colorElement.css('background-color','#'+hex);
                    $colorElement.find(".color").css("background-color",'#'+hex).val('#'+hex);
                    self._chooseColor(hsb,hex,rgb,$colorElement.find(".color")[0]);
                    $colorElement.val(hex);

                };
                blr.colorPicker.onClose = function(hex){
                    //   var oComponent = eveManager.getSelectComponent().getData();
                    //  $.publish("/model/components/"+ oComponent.id +"/update/parameter",[oComponent,'color-1',"#"+hex]);
                    //   $.publish("/view/components/"+ oComponent.id +"/refresh/parameter",[oComponent,'color-1']);
                };
                blr.colorPicker.show($colorElement,bigh.textRgb2hex($(this).css("background-color")));
            });

/*
            $element.find(".value").css("background-color",oDescriptor.getDefaultValue().replace("0x","#")).colpick({
                onBeforeShow: function(pickerElem){
                    //Background color described as "rgb(R,G,B)"
                    //TODO check "bigh.textRgb2hex" function
                    console.log("before show");
                    console.log(bigh.textRgb2hex($(this).css("background-color")));
                    //console.error($(this).css("background-color"));
                    //$(this).ColorPickerSetColor(bigh.textRgb2hex($(this).css("background-color")));
                    //$(this).colpickSetColor(bigh.textRgb2hex($(this).css("background-color")));
                    $(this).colpickSetColor(bigh.textRgb2hex($(this).css("background-color")));

                    //console.error(pickerElem);
                    //$(this).css('border-color',bigh.textRgb2hex($(this).css("background-color")));
                },
                //On choose color
                layout:'hex',
                //onSubmit: self._chooseColor,
                colorScheme:'dark',
                submit:false,
                //flat: true,
                //onChange: self._chooseColor
                //onChange: function(param,hex,hue){
                onChange: function(hsb,hex,rgb,el,bySetColor){


                    var hexColor = hex;
                    //$(this).find(".colorpicker_current_color")
                    $element.find(".value").css( "background-color","#"+hexColor );
                    $element.find(".value").val("0x"+hexColor);

                }
            }).keyup(function(){
                $(this).colpickSetColor(this.value);
            });

*/



        }
        
        //Enable/disable actions selector
        if(oDescriptor.isActionEditable()){
            $element.find(".actions").removeProp("disabled");
        }
        else{
            $element.find(".actions").prop("disabled",true);
        }
        
        //Enable/disable value field
        if(oDescriptor.isValueEditable()){
            $element.find(".value").show();
        }
        else{
            $element.find(".value").hide();
        }
        
        var $componentActions = $element.find(".actions");
        var oActionDescription = oDescriptor.getDescriptions();
        
        //Append options
        for(var k in oActionDescription){
            var $option = $("<option></option>");
            $option.val(k);
            $option.text(oActionDescription[k]);
            $componentActions.append($option);
        }
        
        //Set default action
        var iAction = oDescriptor.getDefaultAction();
        $componentActions.find("[value='"+ iAction +"']").prop("selected",true);
        
        $element.data("eveManagerNode",oNode);                                                      //Set node data with html element
        
        return $element;
        /*$helper.before($element);                                                                   //Append html element
        
        //Remove helper
        $helper.remove();*/
    };

    /** Chose color
     *
     */
    this._chooseColor = function(param,hex,hue,HTMLelem){



        //Change item background color
        $(HTMLelem).css("background-color","#"+ hex);
        //$(HTMLelem).css("border-right-color","#"+ hex);

        //m_$element.find(".color").css("border-right","#"+ hex);

        //m_$element.find();
    };
    /** Confirm changes
     * 
     */
    this._submit = function(e){
        //Call external window hide method
        m_oWindow.hide();
    };
}