/** Javascript Aernova Wizard TU01 TU02 Importer
 *
 * 28/8/2015 by Andreosky Padovanosky
 *
 */
"use strict";

var VERSION = "v.1.0.1";
var NAME = "Aernova TU01 TU02 Wizard " + VERSION;
var $ = jQuery;


try {
    console.log("%cStart " + NAME, "color:DarkSlateBlue");
    create_gui();
    console.log("%cEnd " + NAME, "color:DarkSlateBlue");
} catch (err) {
    console.error(NAME + " error\n" + err);
    alert(NAME + " error\n" + err);
}


function create_gui() {
    var wizard = $("<div id='aernova-wizard' title='" + NAME + "'></div>");
    wizard.append("</br><label>Name</label></br>");
    wizard.append("<input type='text' id='gtw-name' value='Aernova TU01-TU02'/></br>");
    wizard.append("</br><label>Address</label></br>");
    wizard.append("<input type='text' id='gtw-address' value='192.168.1.250'/></br>");
    wizard.append("</br><label>Port</label></br>");
    wizard.append("<input type='text' id='gtw-port' value='502' onkeypress='bigh.onlyPosInt();' /></br>");
    wizard.append("</br><label>Numbers of zones</label></br>");
    wizard.append("<input type='text' id='zones-count' value='10' onkeypress='bigh.onlyPosInt();' /></br>");
    $("body").append(wizard);

    wizard.dialog({
        modal: true,
        width: 350,
        close: function () {
            $(this).remove();
        },
        buttons: {
            "Create Componens...": function () {
                $(this).dialog("close");
                $(this).remove();

                blr.executionStack.show(NAME);
                main(
                    $(this).children("#gtw-name").val(),
                    $(this).children("#gtw-address").val(),
                    $(this).children("#gtw-port").val(),
                    $(this).children("#zones-count").val()
                );
            },
            Cancel: function () {
                $(this).dialog("close");
                $(this).remove();
            }
        }
    });

    return wizard;
}

function main(name, ip, port, zonesCount) {
    try {
        var oGateway = getGateway(name, ip, port);
        var oGroup = getMainGroup(name);

        var oVisitor = new blrComponentsVisitor();

        for (var i = 0; i < zonesCount; ++i) {
            var zoneID = i+1;
            blr.executionStack.push(
                function (in_args, out_args) {
                    out_args = create_device(in_args.visitor,in_args.gateway, in_args.group, in_args.slaveID, in_args.zoneID);
                },
                {
                    visitor: oVisitor,
                    gateway: oGateway,
                    group: oGroup,
                    slaveID: zoneID,
                    zoneID: zoneID
                }
            );
        }

        blr.executionStack.execute(10, true);
    } catch (err) {
        console.error(NAME + " error\n" + err);
        alert(NAME + " error\n" + err);
    }
}

function create_device(visitor,gateway, group, slaveID, zoneID) {
    var oItem = null;
    var oGroup = application.addComponent({
        type: "group",
        name: _D("Zone") + " " + zoneID
    }, group);

    // O Info
    var oGroup0Info = application.addComponent({
        type: "group",
        name: _D("General Info")
    }, oGroup);

    var oItem = undefined;
    try{
        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroup0Info,
            _D("Thermostat Type"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 0,
                "modbusrtutcp/read-bit-count": 2,
                "modbusrtutcp/read-transform-type": 2 // byte
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroup0Info,
            _D("Thermostat Keypad Enable"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 2,
                "modbusrtutcp/read-bit-count": 1,
                "modbusrtutcp/read-transform-type": 1 // bit
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroup0Info,
            _D("Thermostat Power"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 3,
                "modbusrtutcp/read-bit-count": 1,
                "modbusrtutcp/read-transform-type": 1 // bit
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroup0Info,
            _D("Thermostat Cold"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 4,
                "modbusrtutcp/read-bit-count": 1,
                "modbusrtutcp/read-transform-type": 1 // bit
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroup0Info,
            _D("Thermostat Night"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 5,
                "modbusrtutcp/read-bit-count": 1,
                "modbusrtutcp/read-transform-type": 1 // bit
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroup0Info,
            _D("Thermostat SetPoint Reached"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 6,
                "modbusrtutcp/read-bit-count": 1,
                "modbusrtutcp/read-transform-type": 1 // bit
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroup0Info,
            _D("Thrmostat Manual"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 7,
                "modbusrtutcp/read-bit-count": 1,
                "modbusrtutcp/read-transform-type": 1 // bit
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroup0Info,
            _D("Thermostat Windows Contact"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 8,
                "modbusrtutcp/read-bit-count": 1,
                "modbusrtutcp/read-transform-type": 1 // bit
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroup0Info,
            _D("Thermostat Energy Saving"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 9,
                "modbusrtutcp/read-bit-count": 1,
                "modbusrtutcp/read-transform-type": 1 // bit
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroup0Info,
            _D("Thermostat Block"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 10,
                "modbusrtutcp/read-bit-count": 1,
                "modbusrtutcp/read-transform-type": 1 // bit
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroup0Info,
            _D("Thermostat Presence Contact"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 11,
                "modbusrtutcp/read-bit-count": 1,
                "modbusrtutcp/read-transform-type": 1 // bit
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroup0Info,
            _D("Motor Speed"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 16,
                "modbusrtutcp/read-bit-count": 8,
                "modbusrtutcp/read-transform-type": 2 // byte
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroup0Info,
            _D("Current Temperature"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 32,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 5,// Int16
                "modbusrtutcp/read-transform-multiplier" : 0.1,
                "modbusrtutcp/read-transform-digits" : 1,
                "modbusrtutcp/read-transform-digits-format" : 1
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroup0Info,
            _D("Setpoint Temperature"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 48,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 5,// Int16
                "modbusrtutcp/read-transform-multiplier" : 0.1,
                "modbusrtutcp/read-transform-digits" : 1,
                "modbusrtutcp/read-transform-digits-format" : 1
            }
        );

        // 7-509 Temperatures
        var oGroup7_509Temperatures = application.addComponent({
            type: "group",
            name: _D("Info Temperatures")
        }, oGroup);

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroup7_509Temperatures,
            _D("Setpoint if 4Tubes, hot otherwise"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 112,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 5,// Int16
                "modbusrtutcp/read-transform-multiplier" : 0.1,
                "modbusrtutcp/read-transform-digits" : 1,
                "modbusrtutcp/read-transform-digits-format" : 1
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroup7_509Temperatures,
            _D("Setpoint Cold if no 4Tubes"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 128,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 5,// Int16
                "modbusrtutcp/read-transform-multiplier" : 0.1,
                "modbusrtutcp/read-transform-digits" : 1,
                "modbusrtutcp/read-transform-digits-format" : 1
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroup7_509Temperatures,
            _D("Setpoint Center"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 144,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 5,// Int16
                "modbusrtutcp/read-transform-multiplier" : 0.1,
                "modbusrtutcp/read-transform-digits" : 1,
                "modbusrtutcp/read-transform-digits-format" : 1
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroup7_509Temperatures,
            _D("Setpoint Offset"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 160,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 5,// Int16
                "modbusrtutcp/read-transform-multiplier" : 0.1,
                "modbusrtutcp/read-transform-digits" : 1,
                "modbusrtutcp/read-transform-digits-format" : 1
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroup7_509Temperatures,
            _D("Setpoint speed"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 176,
                "modbusrtutcp/read-bit-count": 8,
                "modbusrtutcp/read-transform-type": 2 // Byte
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroup7_509Temperatures,
            _D("4Tubes"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 197,
                "modbusrtutcp/read-bit-count": 1,
                "modbusrtutcp/read-transform-type": 1 // Bit
            }
        );

        // 24-526 Aux States
        var oGroup24_526AuxStates = application.addComponent({
            type: "group",
            name: _D("Info Aux States")
        }, oGroup);

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroup24_526AuxStates,
            _D("Temperature Night Delta"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 384,
                "modbusrtutcp/read-bit-count": 8,
                "modbusrtutcp/read-transform-type": 2, // Byte
                "modbusrtutcp/read-transform-multiplier" : 0.1,
                "modbusrtutcp/read-transform-digits" : 1,
                "modbusrtutcp/read-transform-digits-format" : 1
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroup24_526AuxStates,
            _D("Temperature Presence Delta"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 400,
                "modbusrtutcp/read-bit-count": 8,
                "modbusrtutcp/read-transform-type": 2, // Byte
                "modbusrtutcp/read-transform-multiplier" : 0.1,
                "modbusrtutcp/read-transform-digits" : 1,
                "modbusrtutcp/read-transform-digits-format" : 1
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroup24_526AuxStates,
            _D("Speed Motor Min"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 416,
                "modbusrtutcp/read-bit-count": 8,
                "modbusrtutcp/read-transform-type": 2 // Byte
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroup24_526AuxStates,
            _D("Speed Motor Max"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 432,
                "modbusrtutcp/read-bit-count": 8,
                "modbusrtutcp/read-transform-type": 2 // Byte
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroup24_526AuxStates,
            _D("Presence Contact Enable"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 464,
                "modbusrtutcp/read-bit-count": 1,
                "modbusrtutcp/read-transform-type": 1 // Bit
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroup24_526AuxStates,
            _D("Window Contact Enable"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 466,
                "modbusrtutcp/read-bit-count": 1,
                "modbusrtutcp/read-transform-type": 1 // Bit
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroup24_526AuxStates,
            _D("Window Contact State"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 467,
                "modbusrtutcp/read-bit-count": 2,
                "modbusrtutcp/read-transform-type": 2 // Byte
            }
        );

        // 38-46-63-71-84-90-104-122 Board Parameters
        var oGroupBoardParameters = application.addComponent({
            type: "group",
            name: _D("Info Board Parameters")
        }, oGroup);

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroupBoardParameters,
            _D("Water Hot Min Threshold"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 608,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 4, // Word
                "modbusrtutcp/read-transform-multiplier" : 0.1,
                "modbusrtutcp/read-transform-digits" : 1,
                "modbusrtutcp/read-transform-digits-format" : 1
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroupBoardParameters,
            _D("Water Cold Max Threshold"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 624,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 4, // Word
                "modbusrtutcp/read-transform-multiplier" : 0.1,
                "modbusrtutcp/read-transform-digits" : 1,
                "modbusrtutcp/read-transform-digits-format" : 1
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroupBoardParameters,
            _D("Water Hot Min Enable"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 640,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 1 // bit
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroupBoardParameters,
            _D("Water Cold Max Threshold Value"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 656,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 2 // byte
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroupBoardParameters,
            _D("Windows Contat Manage Mode"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 736,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 2 // byte
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroupBoardParameters,
            _D("Setpoint Umidity"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 1008,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 2 // byte
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroupBoardParameters,
            _D("0/10V Min Value"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 1136,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 2, // byte
                "modbusrtutcp/read-transform-multiplier" : 0.1,
                "modbusrtutcp/read-transform-digits" : 1,
                "modbusrtutcp/read-transform-digits-format" : 1
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroupBoardParameters,
            _D("0/10V Max Value"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 1152,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 2, // byte
                "modbusrtutcp/read-transform-multiplier" : 0.1,
                "modbusrtutcp/read-transform-digits" : 1,
                "modbusrtutcp/read-transform-digits-format" : 1
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroupBoardParameters,
            _D("0/10V Out Ramp"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 1168,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 2, // byte
                "modbusrtutcp/read-transform-multiplier" : 0.1,
                "modbusrtutcp/read-transform-digits" : 1,
                "modbusrtutcp/read-transform-digits-format" : 1
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroupBoardParameters,
            _D("Presence Contact Mode"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 1344,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 1 // bit
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroupBoardParameters,
            _D("Temperature Delta Customer"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 1376,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 2, // byte
                "modbusrtutcp/read-transform-multiplier" : 0.1,
                "modbusrtutcp/read-transform-digits" : 1,
                "modbusrtutcp/read-transform-digits-format" : 1
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroupBoardParameters,
            _D("Umidity Manage Enable"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 1440,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 1 // bit
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroupBoardParameters,
            _D("Hot Floor Enable"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 1456,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 1 // bit
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroupBoardParameters,
            _D("Cold Floor Enable"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 1472,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 1 // bit
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroupBoardParameters,
            _D("Umidity Mode"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 1664,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 2 // byte
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroupBoardParameters,
            _D("Deumidity Enable"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 1680,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 1 // bit
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroupBoardParameters,
            _D("Temperature Remote Probe"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 122,
                "modbusrtutcp/read-register-count": 6,
                "modbusrtutcp/read-bit-start": 0,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 5, // Int16
                "modbusrtutcp/read-transform-multiplier" : 0.1,
                "modbusrtutcp/read-transform-digits" : 1,
                "modbusrtutcp/read-transform-digits-format" : 1
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroupBoardParameters,
            _D("Temperature Remote Probe Max"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 122,
                "modbusrtutcp/read-register-count": 6,
                "modbusrtutcp/read-bit-start": 16,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 5, // Int16
                "modbusrtutcp/read-transform-multiplier" : 0.1,
                "modbusrtutcp/read-transform-digits" : 1,
                "modbusrtutcp/read-transform-digits-format" : 1
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroupBoardParameters,
            _D("Temperature Remote Probe Min"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 122,
                "modbusrtutcp/read-register-count": 6,
                "modbusrtutcp/read-bit-start": 32,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 5, // Int16
                "modbusrtutcp/read-transform-multiplier" : 0.1,
                "modbusrtutcp/read-transform-digits" : 1,
                "modbusrtutcp/read-transform-digits-format" : 1
            }
        );

        oItem = getOrCreateInfo(
            visitor,
            gateway,
            oGroupBoardParameters,
            _D("Temperature Remote Probe Post Hot"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 122,
                "modbusrtutcp/read-register-count": 6,
                "modbusrtutcp/read-bit-start": 80,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 5, // Int16
                "modbusrtutcp/read-transform-multiplier" : 0.1,
                "modbusrtutcp/read-transform-digits" : 1,
                "modbusrtutcp/read-transform-digits-format" : 1
            }
        );

        // Set Temperatures & Speeds
        var oGrouptemperaturesAndSpeeds = application.addComponent({
            type: "group",
            name: _D("Set Temperatures & Speeds")
        }, oGroup);

        oItem = getOrCreateThermostat(
            visitor,
            gateway,
            oGrouptemperaturesAndSpeeds,
            _D("Setpoint if 4Tubes, hot otherwise"),
            {
                "gateway":gateway,
                "step":0.1,
                "min":5,
                "max":35,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 112,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 4, // Word
                "modbusrtutcp/read-transform-min": 50,
                "modbusrtutcp/read-transform-max": 350,
                "modbusrtutcp/read-transform-multiplier" : 0.1,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 7,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": -1,
                "modbusrtutcp/write-bit-count": 0,
                "modbusrtutcp/write-transform-type": 4, // word
                "modbusrtutcp/write-transform-min": 50,
                "modbusrtutcp/write-transform-max": 350,
                "modbusrtutcp/write-transform-multiplier": 0.1
            }
        );

        oItem = getOrCreateThermostat(
            visitor,
            gateway,
            oGrouptemperaturesAndSpeeds,
            _D("Setpoint Cold if no 4Tubes"),
            {
                "gateway":gateway,
                "step":0.1,
                "min":5,
                "max":35,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 128,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 4, // Word
                "modbusrtutcp/read-transform-min": 50,
                "modbusrtutcp/read-transform-max": 350,
                "modbusrtutcp/read-transform-multiplier" : 0.1,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 8,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": -1,
                "modbusrtutcp/write-bit-count": 0,
                "modbusrtutcp/write-transform-type": 4, // word
                "modbusrtutcp/write-transform-min": 50,
                "modbusrtutcp/write-transform-max": 350,
                "modbusrtutcp/write-transform-multiplier": 0.1
            }
        );

        oItem = getOrCreateThermostat(
            visitor,
            gateway,
            oGrouptemperaturesAndSpeeds,
            _D("Setpoint Center"),
            {
                "gateway":gateway,
                "step":0.1,
                "min":5,
                "max":35,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 144,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 4, // Word
                "modbusrtutcp/read-transform-min": 50,
                "modbusrtutcp/read-transform-max": 350,
                "modbusrtutcp/read-transform-multiplier" : 0.1,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 9,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": -1,
                "modbusrtutcp/write-bit-count": 0,
                "modbusrtutcp/write-transform-type": 4, // word
                "modbusrtutcp/write-transform-min": 50,
                "modbusrtutcp/write-transform-max": 350,
                "modbusrtutcp/write-transform-multiplier": 0.1
            }
        );

        oItem = getOrCreateThermostat(
            visitor,
            gateway,
            oGrouptemperaturesAndSpeeds,
            _D("Setpoint Offset"),
            {
                "gateway":gateway,
                "step":0.1,
                "min":0,
                "max":5,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 160,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 4, // Word
                "modbusrtutcp/read-transform-min": 0,
                "modbusrtutcp/read-transform-max": 50,
                "modbusrtutcp/read-transform-multiplier" : 0.1,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 10,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": -1,
                "modbusrtutcp/write-bit-count": 0,
                "modbusrtutcp/write-transform-type": 4, // word
                "modbusrtutcp/write-transform-min": 0,
                "modbusrtutcp/write-transform-max": 50,
                "modbusrtutcp/write-transform-multiplier": 0.1
            }
        );

        oItem = getOrCreateThermostat(
            visitor,
            gateway,
            oGrouptemperaturesAndSpeeds,
            _D("Temperature Night Delta"),
            {
                "gateway":gateway,
                "step":0.1,
                "min":1,
                "max":10,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 384,
                "modbusrtutcp/read-bit-count": 8,
                "modbusrtutcp/read-transform-type": 4, // Word
                "modbusrtutcp/read-transform-min": 1,
                "modbusrtutcp/read-transform-max": 100,
                "modbusrtutcp/read-transform-multiplier" : 0.1,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 24,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": -1,
                "modbusrtutcp/write-bit-count": 0,
                "modbusrtutcp/write-transform-type": 4, // word
                "modbusrtutcp/write-transform-min": 1,
                "modbusrtutcp/write-transform-max": 100,
                "modbusrtutcp/write-transform-multiplier": 0.1
            }
        );

        oItem = getOrCreateThermostat(
            visitor,
            gateway,
            oGrouptemperaturesAndSpeeds,
            _D("Temperature Presence Delta"),
            {
                "gateway":gateway,
                "step":0.1,
                "min":0,
                "max":10,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 400,
                "modbusrtutcp/read-bit-count": 8,
                "modbusrtutcp/read-transform-type": 4, // Word
                "modbusrtutcp/read-transform-min": 0,
                "modbusrtutcp/read-transform-max": 100,
                "modbusrtutcp/read-transform-multiplier" : 0.1,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 25,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": -1,
                "modbusrtutcp/write-bit-count": 0,
                "modbusrtutcp/write-transform-type": 4, // word
                "modbusrtutcp/write-transform-min": 0,
                "modbusrtutcp/write-transform-max": 100,
                "modbusrtutcp/write-transform-multiplier": 0.1
            }
        );

        oItem = getOrCreateThermostat(
            visitor,
            gateway,
            oGrouptemperaturesAndSpeeds,
            _D("Water Hot Min Threshold"),
            {
                "gateway":gateway,
                "step":0.1,
                "min":0,
                "max":60,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 608,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 4, // Word
                "modbusrtutcp/read-transform-min": 0,
                "modbusrtutcp/read-transform-max": 600,
                "modbusrtutcp/read-transform-multiplier" : 0.1,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 38,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": -1,
                "modbusrtutcp/write-bit-count": 0,
                "modbusrtutcp/write-transform-type": 4, // word
                "modbusrtutcp/write-transform-min": 0,
                "modbusrtutcp/write-transform-max": 600,
                "modbusrtutcp/write-transform-multiplier": 0.1
            }
        );

        oItem = getOrCreateThermostat(
            visitor,
            gateway,
            oGrouptemperaturesAndSpeeds,
            _D("Water Cold Max Threshold"),
            {
                "gateway":gateway,
                "step":0.1,
                "min":5,
                "max":30,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 624,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 4, // Word
                "modbusrtutcp/read-transform-min": 50,
                "modbusrtutcp/read-transform-max": 300,
                "modbusrtutcp/read-transform-multiplier" : 0.1,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 39,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": -1,
                "modbusrtutcp/write-bit-count": 0,
                "modbusrtutcp/write-transform-type": 4, // word
                "modbusrtutcp/write-transform-min": 50,
                "modbusrtutcp/write-transform-max": 300,
                "modbusrtutcp/write-transform-multiplier": 0.1
            }
        );

        oItem = getOrCreateThermostat(
            visitor,
            gateway,
            oGrouptemperaturesAndSpeeds,
            _D("Temperature Delta Customer"),
            {
                "gateway":gateway,
                "step":0.1,
                "min":0,
                "max":5,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 1376,
                "modbusrtutcp/read-bit-count": 8,
                "modbusrtutcp/read-transform-type": 4, // Word
                "modbusrtutcp/read-transform-min": 0,
                "modbusrtutcp/read-transform-max": 50,
                "modbusrtutcp/read-transform-multiplier" : 0.1,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 86,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": -1,
                "modbusrtutcp/write-bit-count": 0,
                "modbusrtutcp/write-transform-type": 4, // word
                "modbusrtutcp/write-transform-min": 0,
                "modbusrtutcp/write-transform-max": 50,
                "modbusrtutcp/write-transform-multiplier": 0.1
            }
        );

        // Tuners
        oItem = getOrCreateTuner(
            visitor,
            gateway,
            oGrouptemperaturesAndSpeeds,
            _D("Setpoint speed"),
            {
                "gateway":gateway,
                "min":1,
                "max":100,
                "step":1,
                "digits":0,
                "suffix":"%",
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 176,
                "modbusrtutcp/read-bit-count": 8,
                "modbusrtutcp/read-transform-type": 4, // Word
                "modbusrtutcp/read-transform-min": 1,
                "modbusrtutcp/read-transform-max": 100,
                "modbusrtutcp/read-transform-multiplier" : 1,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 11,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": -1,
                "modbusrtutcp/write-bit-count": 0,
                "modbusrtutcp/write-transform-type": 2, // byte
                "modbusrtutcp/write-transform-min": 1,
                "modbusrtutcp/write-transform-max": 100,
                "modbusrtutcp/write-transform-multiplier": 1
            }
        );

        oItem = getOrCreateTuner(
            visitor,
            gateway,
            oGrouptemperaturesAndSpeeds,
            _D("Speed Motor Min"),
            {
                "gateway":gateway,
                "min":1,
                "max":100,
                "step":1,
                "digits":0,
                "suffix":"%",
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 416,
                "modbusrtutcp/read-bit-count": 8,
                "modbusrtutcp/read-transform-type": 4, // Word
                "modbusrtutcp/read-transform-min": 1,
                "modbusrtutcp/read-transform-max": 100,
                "modbusrtutcp/read-transform-multiplier" : 1,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 26,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": -1,
                "modbusrtutcp/write-bit-count": 0,
                "modbusrtutcp/write-transform-type": 2, // byte
                "modbusrtutcp/write-transform-min": 1,
                "modbusrtutcp/write-transform-max": 100,
                "modbusrtutcp/write-transform-multiplier": 1
            }
        );

        oItem = getOrCreateTuner(
            visitor,
            gateway,
            oGrouptemperaturesAndSpeeds,
            _D("Speed Motor Max"),
            {
                "gateway":gateway,
                "min":1,
                "max":100,
                "step":1,
                "digits":0,
                "suffix":"%",
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 432,
                "modbusrtutcp/read-bit-count": 8,
                "modbusrtutcp/read-transform-type": 4, // Word
                "modbusrtutcp/read-transform-min": 1,
                "modbusrtutcp/read-transform-max": 100,
                "modbusrtutcp/read-transform-multiplier" : 1,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 27,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": -1,
                "modbusrtutcp/write-bit-count": 0,
                "modbusrtutcp/write-transform-type": 2, // byte
                "modbusrtutcp/write-transform-min": 1,
                "modbusrtutcp/write-transform-max": 100,
                "modbusrtutcp/write-transform-multiplier": 1
            }
        );

        oItem = getOrCreateTuner(
            visitor,
            gateway,
            oGrouptemperaturesAndSpeeds,
            _D("Setpoint Umidity"),
            {
                "gateway":gateway,
                "min":10,
                "max":90,
                "step":1,
                "digits":0,
                "suffix":"%",
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 1008,
                "modbusrtutcp/read-bit-count": 8,
                "modbusrtutcp/read-transform-type": 2, // byte
                "modbusrtutcp/read-transform-min": 10,
                "modbusrtutcp/read-transform-max": 90,
                "modbusrtutcp/read-transform-multiplier" : 1,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 63,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": -1,
                "modbusrtutcp/write-bit-count": 0,
                "modbusrtutcp/write-transform-type": 2, // byte
                "modbusrtutcp/write-transform-min": 10,
                "modbusrtutcp/write-transform-max": 90,
                "modbusrtutcp/write-transform-multiplier": 1
            }
        );

        oItem = getOrCreateTuner(
            visitor,
            gateway,
            oGrouptemperaturesAndSpeeds,
            _D("0/10V Min Value"),
            {
                "gateway":gateway,
                "min":0,
                "max":10,
                "step":0.1,
                "digits":1,
                "suffix":"V",
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 1136,
                "modbusrtutcp/read-bit-count": 8,
                "modbusrtutcp/read-transform-type": 2, // byte
                "modbusrtutcp/read-transform-min": 0,
                "modbusrtutcp/read-transform-max": 100,
                "modbusrtutcp/read-transform-multiplier" : 0.1,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 71,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": -1,
                "modbusrtutcp/write-bit-count": 0,
                "modbusrtutcp/write-transform-type": 2, // byte
                "modbusrtutcp/write-transform-min": 0,
                "modbusrtutcp/write-transform-max": 100,
                "modbusrtutcp/write-transform-multiplier": 0.1
            }
        );

        oItem = getOrCreateTuner(
            visitor,
            gateway,
            oGrouptemperaturesAndSpeeds,
            _D("0/10V Max Value"),
            {
                "gateway":gateway,
                "min":0,
                "max":10,
                "step":0.1,
                "digits":1,
                "suffix":"V",
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 1152,
                "modbusrtutcp/read-bit-count": 8,
                "modbusrtutcp/read-transform-type": 2, // byte
                "modbusrtutcp/read-transform-min": 0,
                "modbusrtutcp/read-transform-max": 100,
                "modbusrtutcp/read-transform-multiplier" : 0.1,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 72,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": -1,
                "modbusrtutcp/write-bit-count": 0,
                "modbusrtutcp/write-transform-type": 2, // byte
                "modbusrtutcp/write-transform-min": 0,
                "modbusrtutcp/write-transform-max": 100,
                "modbusrtutcp/write-transform-multiplier": 0.1
            }
        );

        oItem = getOrCreateTuner(
            visitor,
            gateway,
            oGrouptemperaturesAndSpeeds,
            _D("0/10V Out Ramp"),
            {
                "gateway":gateway,
                "min":0,
                "max":10,
                "step":0.1,
                "digits":1,
                "suffix":"V",
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 1168,
                "modbusrtutcp/read-bit-count": 8,
                "modbusrtutcp/read-transform-type": 2, // byte
                "modbusrtutcp/read-transform-min": 0,
                "modbusrtutcp/read-transform-max": 100,
                "modbusrtutcp/read-transform-multiplier" : 0.1,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 73,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": -1,
                "modbusrtutcp/write-bit-count": 0,
                "modbusrtutcp/write-transform-type": 2, // byte
                "modbusrtutcp/write-transform-min": 0,
                "modbusrtutcp/write-transform-max": 100,
                "modbusrtutcp/write-transform-multiplier": 0.1
            }
        );

        oItem = getOrCreateSlider(
            visitor,
            gateway,
            oGrouptemperaturesAndSpeeds,
            _D("Setpoint speed"),
            {
                "gateway":gateway,
                "min":1,
                "max":100,
                "step":1,
                "digits":0,
                "suffix":"%",
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 176,
                "modbusrtutcp/read-bit-count": 8,
                "modbusrtutcp/read-transform-type": 2, // byte
                "modbusrtutcp/read-transform-min": 1,
                "modbusrtutcp/read-transform-max": 100,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 11,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": -1,
                "modbusrtutcp/write-bit-count": 0,
                "modbusrtutcp/write-transform-type": 2, // byte
                "modbusrtutcp/write-transform-min": 1,
                "modbusrtutcp/write-transform-max": 100
            }
        );

        // Set States
        var oGroupStates = application.addComponent({
            type: "group",
            name: _D("Set Sates")
        }, oGroup);

        oItem = getOrCreateSwitch(
            visitor,
            gateway,
            oGroupStates,
            _D("Thermostat Power"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 3,
                "modbusrtutcp/read-bit-count": 1,
                "modbusrtutcp/read-transform-type": 1, // bit
                "modbusrtutcp/read-transform-min": 0,
                "modbusrtutcp/read-transform-max": 1,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 12,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": 1,
                "modbusrtutcp/write-bit-count": 1,
                "modbusrtutcp/write-transform-type": 1, // bit
                "modbusrtutcp/write-transform-min": 0,
                "modbusrtutcp/write-transform-max": 1
            }
        );

        oItem = getOrCreateSwitch(
            visitor,
            gateway,
            oGroupStates,
            _D("Thermostat Night"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 5,
                "modbusrtutcp/read-bit-count": 1,
                "modbusrtutcp/read-transform-type": 1, // bit
                "modbusrtutcp/read-transform-min": 0,
                "modbusrtutcp/read-transform-max": 1,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 12,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": 3,
                "modbusrtutcp/write-bit-count": 1,
                "modbusrtutcp/write-transform-type": 1, // bit
                "modbusrtutcp/write-transform-min": 0,
                "modbusrtutcp/write-transform-max": 1
            }
        );

        oItem = getOrCreateSwitch(
            visitor,
            gateway,
            oGroupStates,
            _D("Thermostat Keypad Enable"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 2,
                "modbusrtutcp/read-bit-count": 1,
                "modbusrtutcp/read-transform-type": 1, // bit
                "modbusrtutcp/read-transform-min": 0,
                "modbusrtutcp/read-transform-max": 1,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 12,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": 0,
                "modbusrtutcp/write-bit-count": 1,
                "modbusrtutcp/write-transform-type": 1, // bit
                "modbusrtutcp/write-transform-min": 0,
                "modbusrtutcp/write-transform-max": 1
            }
        );

        oItem = getOrCreateSwitch(
            visitor,
            gateway,
            oGroupStates,
            _D("Presence Contact Enable"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 464,
                "modbusrtutcp/read-bit-count": 1,
                "modbusrtutcp/read-transform-type": 1, // bit
                "modbusrtutcp/read-transform-min": 0,
                "modbusrtutcp/read-transform-max": 1,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 29,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": 0,
                "modbusrtutcp/write-bit-count": 1,
                "modbusrtutcp/write-transform-type": 1, // bit
                "modbusrtutcp/write-transform-min": 0,
                "modbusrtutcp/write-transform-max": 1
            }
        );

        oItem = getOrCreateSwitch(
            visitor,
            gateway,
            oGroupStates,
            _D("Window Contact Enable"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 466,
                "modbusrtutcp/read-bit-count": 1,
                "modbusrtutcp/read-transform-type": 1, // bit
                "modbusrtutcp/read-transform-min": 0,
                "modbusrtutcp/read-transform-max": 1,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 29,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": 2,
                "modbusrtutcp/write-bit-count": 1,
                "modbusrtutcp/write-transform-type": 1, // bit
                "modbusrtutcp/write-transform-min": 0,
                "modbusrtutcp/write-transform-max": 1
            }
        );

        oItem = getOrCreateSwitch(
            visitor,
            gateway,
            oGroupStates,
            _D("4Tubes"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 197,
                "modbusrtutcp/read-bit-count": 1,
                "modbusrtutcp/read-transform-type": 1, // bit
                "modbusrtutcp/read-transform-min": 0,
                "modbusrtutcp/read-transform-max": 1,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 12,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": 5,
                "modbusrtutcp/write-bit-count": 1,
                "modbusrtutcp/write-transform-type": 1, // bit
                "modbusrtutcp/write-transform-min": 0,
                "modbusrtutcp/write-transform-max": 1
            }
        );

        oItem = getOrCreateSwitch(
            visitor,
            gateway,
            oGroupStates,
            _D("Water Hot Min Enable"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 640,
                "modbusrtutcp/read-bit-count": 1,
                "modbusrtutcp/read-transform-type": 1, // bit
                "modbusrtutcp/read-transform-min": 0,
                "modbusrtutcp/read-transform-max": 1,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 40,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": -1,
                "modbusrtutcp/write-bit-count": 0,
                "modbusrtutcp/write-transform-type": 1, // bit
                "modbusrtutcp/write-transform-min": 0,
                "modbusrtutcp/write-transform-max": 1
            }
        );

        oItem = getOrCreateSwitch(
            visitor,
            gateway,
            oGroupStates,
            _D("Umidity Manage Enable"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 1440,
                "modbusrtutcp/read-bit-count": 1,
                "modbusrtutcp/read-transform-type": 1, // bit
                "modbusrtutcp/read-transform-min": 0,
                "modbusrtutcp/read-transform-max": 1,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 90,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": -1,
                "modbusrtutcp/write-bit-count": 0,
                "modbusrtutcp/write-transform-type": 1, // bit
                "modbusrtutcp/write-transform-min": 0,
                "modbusrtutcp/write-transform-max": 1
            }
        );

        oItem = getOrCreateSwitch(
            visitor,
            gateway,
            oGroupStates,
            _D("Hot Floor Enable"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 1456,
                "modbusrtutcp/read-bit-count": 1,
                "modbusrtutcp/read-transform-type": 1, // bit
                "modbusrtutcp/read-transform-min": 0,
                "modbusrtutcp/read-transform-max": 1,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 91,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": -1,
                "modbusrtutcp/write-bit-count": 0,
                "modbusrtutcp/write-transform-type": 1, // bit
                "modbusrtutcp/write-transform-min": 0,
                "modbusrtutcp/write-transform-max": 1
            }
        );

        oItem = getOrCreateSwitch(
            visitor,
            gateway,
            oGroupStates,
            _D("Cold Floor Enable"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 1472,
                "modbusrtutcp/read-bit-count": 1,
                "modbusrtutcp/read-transform-type": 1, // bit
                "modbusrtutcp/read-transform-min": 0,
                "modbusrtutcp/read-transform-max": 1,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 92,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": -1,
                "modbusrtutcp/write-bit-count": 0,
                "modbusrtutcp/write-transform-type": 1, // bit
                "modbusrtutcp/write-transform-min": 0,
                "modbusrtutcp/write-transform-max": 1
            }
        );

        oItem = getOrCreateSwitch(
            visitor,
            gateway,
            oGroupStates,
            _D("Deumidity Enable"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 1680,
                "modbusrtutcp/read-bit-count": 1,
                "modbusrtutcp/read-transform-type": 1, // bit
                "modbusrtutcp/read-transform-min": 0,
                "modbusrtutcp/read-transform-max": 1,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 105,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": -1,
                "modbusrtutcp/write-bit-count": 0,
                "modbusrtutcp/write-transform-type": 1, // bit
                "modbusrtutcp/write-transform-min": 0,
                "modbusrtutcp/write-transform-max": 1
            }
        );

        var itemData = undefined;
        itemData = getOrCreateCombo(
            visitor,
            gateway,
            oGroupStates,
            _D("Thermostat Cold/Hot"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 4,
                "modbusrtutcp/read-bit-count": 1,
                "modbusrtutcp/read-transform-type": 1, // bit
                "modbusrtutcp/read-transform-min": 0,
                "modbusrtutcp/read-transform-max": 1,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 12,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": 2,
                "modbusrtutcp/write-bit-count": 1,
                "modbusrtutcp/write-transform-type": 1, // bit
                "modbusrtutcp/write-transform-min": 0,
                "modbusrtutcp/write-transform-max": 1
            }
        );
        oItem = itemData.item;
        if(itemData.new) {
            oItem.data.setParamVal("values",
                [{
                    "value": 0,
                    "description": _D("Hot")
                }, {
                    "value": 1,
                    "description": _D("Cold")
                }]
            );

            oItem.data.setParamVal("icons",
                [
                    blr.gfxResource.getIconByTypeName(0, "hot",200),
                    blr.gfxResource.getIconByTypeName(1, "cold",0,0,200)
                ]
            );
        }

        itemData = getOrCreateCombo(
            visitor,
            gateway,
            oGroupStates,
            _D("Thermostat Manual"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 7,
                "modbusrtutcp/read-bit-count": 1,
                "modbusrtutcp/read-transform-type": 1, // bit
                "modbusrtutcp/read-transform-min": 0,
                "modbusrtutcp/read-transform-max": 1,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 12,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": 4,
                "modbusrtutcp/write-bit-count": 1,
                "modbusrtutcp/write-transform-type": 1, // bit
                "modbusrtutcp/write-transform-min": 0,
                "modbusrtutcp/write-transform-max": 1
            }
        );
        oItem = itemData.item;
        if(itemData.new) {
            oItem.data.setParamVal("values",
                [{
                    "value": 0,
                    "description": _D("Automatic")
                }, {
                    "value": 1,
                    "description": _D("Manual")
                }]
            );

            oItem.data.setParamVal("icons",
                [
                    blr.gfxResource.getIconByTypeName(0, "automatic"),
                    blr.gfxResource.getIconByTypeName(1, "manual")
                ]
            );
        }

        itemData = getOrCreateCombo(
            visitor,
            gateway,
            oGroupStates,
            _D("Window Contact State"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 467,
                "modbusrtutcp/read-bit-count": 2,
                "modbusrtutcp/read-transform-type": 2, // byte
                "modbusrtutcp/read-transform-min": 0,
                "modbusrtutcp/read-transform-max": 2,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 29,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": 3,
                "modbusrtutcp/write-bit-count": 2,
                "modbusrtutcp/write-transform-type": 2, // byte
                "modbusrtutcp/write-transform-min": 0,
                "modbusrtutcp/write-transform-max": 2
            }
        );
        oItem = itemData.item;
        if(itemData.new) {
            oItem.data.setParamVal("values",
                [{
                    "value": 0,
                    "description": _D("N.O.")
                }, {
                    "value": 1,
                    "description": _D("N.C.")
                }, {
                    "value": 2,
                    "description": _D("VEN")
                }]
            );

            oItem.data.setParamVal("icons",
                [
                    blr.gfxResource.getIconByTypeName(0, "zero"),
                    blr.gfxResource.getIconByTypeName(1, "one"),
                    blr.gfxResource.getIconByTypeName(1, "two")
                ]
            );
        }

        itemData = getOrCreateCombo(
            visitor,
            gateway,
            oGroupStates,
            _D("Water Cold Max Threshold Value"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 656,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 2, // byte
                "modbusrtutcp/read-transform-min": 0,
                "modbusrtutcp/read-transform-max": 2,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 41,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": -1,
                "modbusrtutcp/write-bit-count": 0,
                "modbusrtutcp/write-transform-type": 2, // byte
                "modbusrtutcp/write-transform-min": 0,
                "modbusrtutcp/write-transform-max": 2
            }
        );
        oItem = itemData.item;
        if(itemData.new) {
            oItem.data.setParamVal("values",
                [{
                    "value": 0,
                    "description": _D("N.O.")
                }, {
                    "value": 1,
                    "description": _D("N.C.")
                }, {
                    "value": 2,
                    "description": _D("VEN")
                }]
            );

            oItem.data.setParamVal("icons",
                [
                    blr.gfxResource.getIconByTypeName(0, "zero"),
                    blr.gfxResource.getIconByTypeName(1, "one"),
                    blr.gfxResource.getIconByTypeName(1, "two")
                ]
            );
        }

        itemData = getOrCreateCombo(
            visitor,
            gateway,
            oGroupStates,
            _D("Windows Contat Manage Mode"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 736,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 2, // byte
                "modbusrtutcp/read-transform-min": 0,
                "modbusrtutcp/read-transform-max": 2,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 46,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": -1,
                "modbusrtutcp/write-bit-count": 0,
                "modbusrtutcp/write-transform-type": 2, // byte
                "modbusrtutcp/write-transform-min": 0,
                "modbusrtutcp/write-transform-max": 2
            }
        );
        oItem = itemData.item;
        if(itemData.new) {
            oItem.data.setParamVal("values",
                [{
                    "value": 0,
                    "description": _D("N.O.")
                }, {
                    "value": 1,
                    "description": _D("N.C.")
                }, {
                    "value": 2,
                    "description": _D("VEN")
                }]
            );

            oItem.data.setParamVal("icons",
                [
                    blr.gfxResource.getIconByTypeName(0, "zero"),
                    blr.gfxResource.getIconByTypeName(1, "one"),
                    blr.gfxResource.getIconByTypeName(1, "two")
                ]
            );
        }

        itemData = getOrCreateCombo(
            visitor,
            gateway,
            oGroupStates,
            _D("Presence Contact Mode"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 1344,
                "modbusrtutcp/read-bit-count": 1,
                "modbusrtutcp/read-transform-type": 1, // bit
                "modbusrtutcp/read-transform-min": 0,
                "modbusrtutcp/read-transform-max": 1,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 84,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": -1,
                "modbusrtutcp/write-bit-count": 0,
                "modbusrtutcp/write-transform-type": 1, // bit
                "modbusrtutcp/write-transform-min": 0,
                "modbusrtutcp/write-transform-max": 1
            }
        );
        oItem = itemData.item;
        if(itemData.new) {
            oItem.data.setParamVal("values",
                [{
                    "value": 0,
                    "description": _D("N.O.")
                }, {
                    "value": 1,
                    "description": _D("N.C.")
                }]
            );

            oItem.data.setParamVal("icons",
                [
                    blr.gfxResource.getIconByTypeName(0, "zero"),
                    blr.gfxResource.getIconByTypeName(1, "one")
                ]
            );
        }

        itemData = getOrCreateCombo(
            visitor,
            gateway,
            oGroupStates,
            _D("Umidity Mode"),
            {
                "gateway":gateway,
                "modbusrtutcp/read-unit-id": slaveID,
                "modbusrtutcp/read-function-id": 3, // (3) Read Holding Registers
                "modbusrtutcp/read-register-start": 0,
                "modbusrtutcp/read-register-count": 106,
                "modbusrtutcp/read-bit-start": 1664,
                "modbusrtutcp/read-bit-count": 16,
                "modbusrtutcp/read-transform-type": 2, // byte
                "modbusrtutcp/read-transform-min": 0,
                "modbusrtutcp/read-transform-max": 2,
                "modbusrtutcp/write-unit-id": slaveID,
                "modbusrtutcp/write-function-id": 6, // (6) Write Single Register
                "modbusrtutcp/write-register-start": 104,
                "modbusrtutcp/write-register-count": 1,
                "modbusrtutcp/write-bit-start": -1,
                "modbusrtutcp/write-bit-count": 0,
                "modbusrtutcp/write-transform-type": 2, // byte
                "modbusrtutcp/write-transform-min": 0,
                "modbusrtutcp/write-transform-max": 2
            }
        );
        oItem = itemData.item;
        if(itemData.new) {
            oItem.data.setParamVal("values",
                [{
                    "value": 0,
                    "description": _D("Disable")
                }, {
                    "value": 1,
                    "description": _D("Minimun")
                }, {
                    "value": 2,
                    "description": _D("Off")
                }]
            );

            oItem.data.setParamVal("icons",
                [
                    blr.gfxResource.getIconByTypeName(0, "disable"),
                    blr.gfxResource.getIconByTypeName(1, "min"),
                    blr.gfxResource.getIconByTypeName(1, "off")
                ]
            );
        }

    } catch (err) {
        blr.executionStack.cancel();
        console.error(NAME + " error\n" + err);
        alert(NAME + " error\n" + err);
    }

    return oGroup;
}

function getOrCreateCombo(visitor,gateway,group,name,parameters){
    var bNew = false;
    var oItem = undefined;
    try{
        oItem = visitor.getComponentBy(
            "modbusrtutcp",
            "combo",
            parameters
        );
        if(oItem == undefined){
            var createParameters = parameters;
            parameters["type"] = "combo";
            parameters["gateway"] = gateway;
            parameters["name"] = name;
            oItem = application.addComponent(createParameters, group);
            bNew = true;
        }
    } catch (err) {
        blr.executionStack.cancel();
        console.error(NAME + " error\n" + err);
        alert(NAME + " error\n" + err);
    }
    return { "item":oItem,"new":bNew };
}

function getOrCreateInfo(visitor,gateway,group,name,parameters){
    var oItem = undefined;
    try{
        oItem = visitor.getComponentBy(
            "modbusrtutcp",
            "info",
            parameters
        );
        if(oItem == undefined){
            var createParameters = parameters;
            parameters["type"] = "info";
            parameters["gateway"] = gateway;
            parameters["name"] = name;
            oItem = application.addComponent(createParameters, group);
        }
    } catch (err) {
        blr.executionStack.cancel();
        console.error(NAME + " error\n" + err);
        alert(NAME + " error\n" + err);
    }
    return oItem;
}

function getOrCreateSwitch(visitor,gateway,group,name,parameters){
    var oItem = undefined;
    try{
        oItem = visitor.getComponentBy(
            "modbusrtutcp",
            "switch",
            parameters
        );
        if(oItem == undefined){
            var createParameters = parameters;
            parameters["type"] = "switch";
            parameters["gateway"] = gateway;
            parameters["name"] = name;
            oItem = application.addComponent(createParameters, group);
        }
    } catch (err) {
        blr.executionStack.cancel();
        console.error(NAME + " error\n" + err);
        alert(NAME + " error\n" + err);
    }
    return oItem;
}

function getOrCreateThermostat(visitor,gateway,group,name,parameters){
    var oItem = undefined;
    try{
        oItem = visitor.getComponentBy(
            "modbusrtutcp",
            "thermostat",
            parameters
        );
        if(oItem == undefined){
            var createParameters = parameters;
            parameters["type"] = "thermostat";
            parameters["gateway"] = gateway;
            parameters["name"] = name;
            oItem = application.addComponent(createParameters, group);
        }
    } catch (err) {
        blr.executionStack.cancel();
        console.error(NAME + " error\n" + err);
        alert(NAME + " error\n" + err);
    }
    return oItem;
}

function getOrCreateTuner(visitor,gateway,group,name,parameters){
    var oItem = undefined;
    try{
        oItem = visitor.getComponentBy(
            "modbusrtutcp",
            "tuner",
            parameters
        );
        if(oItem == undefined){
            var createParameters = parameters;
            parameters["type"] = "tuner";
            parameters["gateway"] = gateway;
            parameters["name"] = name;
            oItem = application.addComponent(createParameters, group);
        }
    } catch (err) {
        blr.executionStack.cancel();
        console.error(NAME + " error\n" + err);
        alert(NAME + " error\n" + err);
    }
    return oItem;
}

function getOrCreateSlider(visitor,gateway,group,name,parameters){
    var oItem = undefined;
    try{
        oItem = visitor.getComponentBy(
            "modbusrtutcp",
            "slider",
            parameters
        );
        if(oItem == undefined){
            var createParameters = parameters;
            parameters["type"] = "slider";
            parameters["gateway"] = gateway;
            parameters["name"] = name;
            oItem = application.addComponent(createParameters, group);
        }
    } catch (err) {
        blr.executionStack.cancel();
        console.error(NAME + " error\n" + err);
        alert(NAME + " error\n" + err);
    }
    return oItem;
}

function getGateway(name, address, port) {
    var bSet = false;
    var oGateway = undefined;

    var aGateways = application.getGatewaysByProtocol("modbusrtutcp");
    for(var g=0; g<aGateways.length; ++g){
        oGateway = aGateways[g];
        if(
            oGateway.data.getParamVal("modbusrtutcp/address") == address &&
            oGateway.data.getParamVal("modbusrtutcp/port") == port
        ){
            bSet = false;
            break;
        }
        oGateway = undefined;
    }

    if (oGateway == undefined) {
        oGateway = application.addGateway({
            name: name,
            protocol: "modbusrtutcp"
        });
        bSet = true;
    }

    if (bSet) {
        oGateway.data.setName(name);
        oGateway.data.setParamVal("modbusrtutcp/address", address);
        oGateway.data.setParamVal("modbusrtutcp/port", port);
        oGateway.data.setParamVal("modbusrtutcp/asciitransform", false);
        oGateway.data.setParamVal("modbusrtutcp/readinterval", 250);
        oGateway.data.setParamVal("modbusrtutcp/timeout", 250);
    }

    return oGateway;
}

function getMainGroup(name) {
    var oGroup = application.addComponent({
        type: "group",
        name: name
    });
    return oGroup;
}
